@@ -560,7 +560,7 @@ static int runsearch(int argc, char **argv){
   depth = 0;
   mask = 0;
   ec = -1.0;
-  pbuf = cbdatumopen("", 0);
+  pbuf = cbdatumopen(NULL, -1);
   cbglobalgc(pbuf, (void (*)(void *))cbdatumclose);
   attrs = cblistopen();
   cbglobalgc(attrs, (void (*)(void *))cblistclose);
@@ -1107,7 +1107,6 @@ static int procoptimize(const char *nurl){
   if(g_timeout >= 0) est_node_set_timeout(node, g_timeout);
   if(g_authname && g_authpass) est_node_set_auth(node, g_authname, g_authpass);
   err = FALSE;
-  
   if(!est_node_optimize(node)){
     printferror("failed: %d", est_node_status(node));
     err = TRUE;
@@ -1424,7 +1423,7 @@ static int procraw(const char *url, const char *file){
   err = FALSE;
   if(g_pxport < 1) g_pxport = 80;
   resheads = cbmapopenex(MINIBNUM);
-  resbody = cbdatumopen("", 0);
+  resbody = cbdatumopen(NULL, -1);
   auth = (g_authname && g_authpass) ? cbsprintf("%s:%s", g_authname, g_authpass) : NULL;
   if(est_url_shuttle(url, g_pxhost, g_pxport, g_timeout, RESLIMSIZE, auth,
                      g_exheaders, reqbody, rbsiz, &rescode, resheads, resbody)){
