@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Implementation of the MT-safe API
- *                                                      Copyright (C) 2004-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2004-2006 Mikio Hirabayashi
  * This file is part of Hyper Estraier.
  * Hyper Estraier is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -119,6 +119,17 @@ int est_mtdb_fatal(ESTMTDB *db){
 }
 
 
+/* Add an index for narrowing or sorting with document attributes. */
+int est_mtdb_add_attr_index(ESTMTDB *db, const char *name, int type){
+  int rv;
+  assert(db && name);
+  if(!est_mtdb_lock(db)) return FALSE;
+  rv = est_db_add_attr_index(db->db, name, type);
+  est_mtdb_unlock(db);
+  return rv;
+}
+
+
 /* Flush index words in the cache of a database. */
 int est_mtdb_flush(ESTMTDB *db, int max){
   int rv;
@@ -152,6 +163,17 @@ int est_mtdb_optimize(ESTMTDB *db, int options){
 }
 
 
+/* Merge another database. */
+int est_mtdb_merge(ESTMTDB *db, const char *name, int options){
+  int rv;
+  assert(db && name);
+  if(!est_mtdb_lock(db)) return FALSE;
+  rv = est_db_merge(db->db, name, options);
+  est_mtdb_unlock(db);
+  return rv;
+}
+
+
 /* Add a document to a database. */
 int est_mtdb_put_doc(ESTMTDB *db, ESTDOC *doc, int options){
   int rv;
@@ -278,6 +300,52 @@ int *est_mtdb_search(ESTMTDB *db, ESTCOND *cond, int *nump, CBMAP *hints){
 }
 
 
+/* Search plural databases for documents corresponding a condition. */
+int *est_mtdb_search_meta(ESTMTDB **dbs, int dbnum, ESTCOND *cond, int *nump, CBMAP *hints){
+  ESTDB **cdbs;
+  CBMAP *umap;
+  const char *kbuf;
+  int i, *rv;
+  assert(dbs && dbnum >= 0 && cond && nump);
+  cdbs = cbmalloc(dbnum * sizeof(ESTDB *) + 1);
+  if(dpisreentrant){
+    umap = cbmapopenex(dbnum + 1);
+    for(i = 0; i < dbnum; i++){
+      if(!cbmapget(umap, (char *)&(dbs[i]), sizeof(ESTDB *), NULL) && !est_mtdb_lock(dbs[i])){
+        while(--i >= 0){
+          est_mtdb_unlock(dbs[i]);
+        }
+        free(cdbs);
+        est_db_set_ecode(cdbs[i], ESTELOCK);
+        cbmapput(hints, "", 0, "0", -1, TRUE);
+        *nump = 0;
+        return cbmalloc(1);
+      }
+      cbmapput(umap, (char *)&(dbs[i]), sizeof(ESTDB *), "", 0, FALSE);
+      cdbs[i] = dbs[i]->db;
+    }
+    rv = est_db_search_meta(cdbs, dbnum, cond, nump, hints);
+    cbmapiterinit(umap);
+    while((kbuf = cbmapiternext(umap, NULL)) != NULL){
+      est_mtdb_unlock(*(ESTMTDB **)kbuf);
+    }
+    cbmapclose(umap);
+  } else {
+    if(pthread_mutex_lock(&est_global_mutex) != 0){
+      *nump = 0;
+      return cbmalloc(1);
+    }
+    for(i = 0; i < dbnum; i++){
+      cdbs[i] = dbs[i]->db;
+    }
+    rv = est_db_search_meta(cdbs, dbnum, cond, nump, hints);
+    pthread_mutex_unlock(&est_global_mutex);
+  }
+  free(cdbs);
+  return rv;
+}
+
+
 /* Check whether a document object matches the phrase of a search condition object definitely. */
 int est_mtdb_scan_doc(ESTMTDB *db, ESTDOC *doc, ESTCOND *cond){
   int rv;
@@ -313,6 +381,17 @@ void est_mtdb_set_ecode(ESTMTDB *db, int ecode){
 }
 
 
+/* Get the inode number of a database. */
+int est_mtdb_inode(ESTMTDB *db){
+  int rv;
+  assert(db);
+  if(!est_mtdb_lock(db)) return -1;
+  rv = est_db_inode(db->db);
+  est_mtdb_unlock(db);
+  return rv;
+}
+
+
 /* Set the entity data of a document in a database. */
 int est_mtdb_set_doc_entity(ESTMTDB *db, int id, const char *ptr, int size){
   int rv;
@@ -335,6 +414,15 @@ char *est_mtdb_get_doc_entity(ESTMTDB *db, int id, int *sp){
 }
 
 
+/* Set the maximum number of extension of wild cards. */
+void est_mtdb_set_wildmax(ESTMTDB *db, int num){
+  assert(db && num >= 0);
+  if(!est_mtdb_lock(db)) return;
+  est_db_set_wildmax(db->db, num);
+  est_mtdb_unlock(db);
+}
+
+
 /* Add a piece of meta data to a database. */
 void est_mtdb_add_meta(ESTMTDB *db, const char *name, const char *value){
   assert(db && name);
@@ -379,11 +467,22 @@ CBMAP *est_mtdb_etch_doc(ESTMTDB *db, ESTDOC *doc, int max){
 
 
 /* Store a map object of keywords. */
-int est_mtdb_put_keywords(ESTMTDB *db, int id, CBMAP *kwords){
+int est_mtdb_put_keywords(ESTMTDB *db, int id, CBMAP *kwords, double weight){
   int rv;
   assert(db && id > 0 && kwords);
   if(!est_mtdb_lock(db)) return FALSE;
-  rv = est_db_put_keywords(db->db, id, kwords);
+  rv = est_db_put_keywords(db->db, id, kwords, weight);
+  est_mtdb_unlock(db);
+  return rv;
+}
+
+
+/* Remove keywords of a document. */
+int est_mtdb_out_keywords(ESTMTDB *db, int id){
+  int rv;
+  assert(db && id > 0);
+  if(!est_mtdb_lock(db)) return FALSE;
+  rv = est_db_out_keywords(db->db, id);
   est_mtdb_unlock(db);
   return rv;
 }
@@ -466,6 +565,61 @@ int est_mtdb_word_rec_size(ESTMTDB *db, const char *word){
 }
 
 
+/* Get the number of unique keywords in a database. */
+int est_mtdb_keyword_num(ESTMTDB *db){
+  int rv;
+  assert(db);
+  if(!est_mtdb_lock(db)) return 0;
+  rv = est_db_keyword_num(db->db);
+  est_mtdb_unlock(db);
+  return rv;
+}
+
+
+/* Initialize the keyword iterator of a database. */
+int est_mtdb_keyword_iter_init(ESTMTDB *db){
+  int rv;
+  assert(db);
+  if(!est_mtdb_lock(db)) return FALSE;
+  rv = est_db_keyword_iter_init(db->db);
+  est_mtdb_unlock(db);
+  return rv;
+}
+
+
+/* Get the next keyword of the word iterator of a database. */
+char *est_mtdb_keyword_iter_next(ESTMTDB *db){
+  char *rv;
+  assert(db);
+  if(!est_mtdb_lock(db)) return NULL;
+  rv = est_db_keyword_iter_next(db->db);
+  est_mtdb_unlock(db);
+  return rv;
+}
+
+
+/* Get the size of the record of a keyword. */
+int est_mtdb_keyword_rec_size(ESTMTDB *db, const char *word){
+  int rv;
+  assert(db && word);
+  if(!est_mtdb_lock(db)) return FALSE;
+  rv = est_db_keyword_rec_size(db->db, word);
+  est_mtdb_unlock(db);
+  return rv;
+}
+
+
+/* Search documents corresponding a keyword for a database. */
+int *est_mtdb_keyword_search(ESTMTDB *db, const char *word, int *nump){
+  int *rv;
+  assert(db && word && nump);
+  if(!est_mtdb_lock(db)) return NULL;
+  rv = est_db_keyword_search(db->db, word, nump);
+  est_mtdb_unlock(db);
+  return rv;
+}
+
+
 /* Get the number of records in the cache memory of a database. */
 int est_mtdb_cache_num(ESTMTDB *db){
   int rv;
@@ -498,10 +652,10 @@ void est_mtdb_set_special_cache(ESTMTDB *db, const char *name, int num){
 
 
 /* Set the callback function to inform of database events. */
-void est_mtdb_set_informer(ESTMTDB *db, void (*func)(const char *)){
+void est_mtdb_set_informer(ESTMTDB *db, void (*func)(const char *, void *), void *opaque){
   assert(db && func);
   if(!est_mtdb_lock(db)) return;
-  est_db_set_informer(db->db, func);
+  est_db_set_informer(db->db, func, opaque);
   est_mtdb_unlock(db);
 }
 
@@ -561,6 +715,19 @@ void est_mtdb_interrupt(ESTMTDB *db){
 
 
 
+/*************************************************************************************************
+ * features for hyper experts
+ *************************************************************************************************/
+
+
+/* Get the usage ratio of the cache. */
+double est_mtdb_cache_usage(ESTMTDB *db){
+  if(db->db->flsflag) return 1.0;
+  return (double)db->db->icsiz / db->db->icmax;
+}
+
+
+
 /*************************************************************************************************
  * private objects
  *************************************************************************************************/
