@@ -422,8 +422,7 @@ int est_node_sync(ESTNODE *node){
     }
   }
   cblistpush(reqheads, "Content-Type: " ESTFORMTYPE, -1);
-  reqbody = cbdatumopen("", 0);
-
+  reqbody = cbdatumopen(NULL, -1);
   if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
                       reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
                       &rescode, NULL, NULL)){
@@ -460,7 +459,7 @@ int est_node_optimize(ESTNODE *node){
     }
   }
   cblistpush(reqheads, "Content-Type: " ESTFORMTYPE, -1);
-  reqbody = cbdatumopen("", 0);
+  reqbody = cbdatumopen(NULL, -1);
   if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
                       reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
                       &rescode, NULL, NULL)){
@@ -532,7 +531,7 @@ int est_node_out_doc(ESTNODE *node, int id){
     }
   }
   cblistpush(reqheads, "Content-Type: " ESTFORMTYPE, -1);
-  reqbody = cbdatumopen("", 0);
+  reqbody = cbdatumopen(NULL, -1);
   est_datum_printf(reqbody, "id=%d", id);
   if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
                       reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
@@ -570,7 +569,7 @@ int est_node_out_doc_by_uri(ESTNODE *node, const char *uri){
     }
   }
   cblistpush(reqheads, "Content-Type: " ESTFORMTYPE, -1);
-  reqbody = cbdatumopen("", 0);
+  reqbody = cbdatumopen(NULL, -1);
   est_datum_printf(reqbody, "uri=%?", uri);
   if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
                       reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
@@ -608,7 +607,7 @@ int est_node_edit_doc(ESTNODE *node, ESTDOC *doc){
     }
   }
   cblistpush(reqheads, "Content-Type: " ESTDRAFTTYPE, -1);
-  reqbody = cbdatumopen("", 0);
+  reqbody = cbdatumopen(NULL, -1);
   names = est_doc_attr_names(doc);
   for(i = 0; i < cblistnum(names); i++){
     kbuf = cblistval(names, i, &ksiz);
@@ -656,9 +655,9 @@ ESTDOC *est_node_get_doc(ESTNODE *node, int id){
     }
   }
   cblistpush(reqheads, "Content-Type: " ESTFORMTYPE, -1);
-  reqbody = cbdatumopen("", 0);
+  reqbody = cbdatumopen(NULL, -1);
   est_datum_printf(reqbody, "id=%d", id);
-  resbody = cbdatumopen("", 0);
+  resbody = cbdatumopen(NULL, -1);
   if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
                       reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
                       &rescode, NULL, resbody)){
@@ -698,9 +697,9 @@ ESTDOC *est_node_get_doc_by_uri(ESTNODE *node, const char *uri){
     }
   }
   cblistpush(reqheads, "Content-Type: " ESTFORMTYPE, -1);
-  reqbody = cbdatumopen("", 0);
+  reqbody = cbdatumopen(NULL, -1);
   est_datum_printf(reqbody, "uri=%?", uri);
-  resbody = cbdatumopen("", 0);
+  resbody = cbdatumopen(NULL, -1);
   if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
                       reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
                       &rescode, NULL, resbody)){
@@ -739,9 +738,9 @@ char *est_node_get_doc_attr(ESTNODE *node, int id, const char *name){
     }
   }
   cblistpush(reqheads, "Content-Type: " ESTFORMTYPE, -1);
-  reqbody = cbdatumopen("", 0);
+  reqbody = cbdatumopen(NULL, -1);
   est_datum_printf(reqbody, "id=%d&attr=%?", id, name);
-  resbody = cbdatumopen("", 0);
+  resbody = cbdatumopen(NULL, -1);
   if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
                       reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
                       &rescode, NULL, resbody)){
@@ -785,9 +784,9 @@ char *est_node_get_doc_attr_by_uri(ESTNODE *node, const char *uri, const char *n
     }
   }
   cblistpush(reqheads, "Content-Type: " ESTFORMTYPE, -1);
-  reqbody = cbdatumopen("", 0);
+  reqbody = cbdatumopen(NULL, -1);
   est_datum_printf(reqbody, "uri=%?&attr=%?", uri, name);
-  resbody = cbdatumopen("", 0);
+  resbody = cbdatumopen(NULL, -1);
   if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
                       reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
                       &rescode, NULL, resbody)){
@@ -833,9 +832,9 @@ CBMAP *est_node_etch_doc(ESTNODE *node, int id){
     }
   }
   cblistpush(reqheads, "Content-Type: " ESTFORMTYPE, -1);
-  reqbody = cbdatumopen("", 0);
+  reqbody = cbdatumopen(NULL, -1);
   est_datum_printf(reqbody, "id=%d", id);
-  resbody = cbdatumopen("", 0);
+  resbody = cbdatumopen(NULL, -1);
   if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
                       reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
                       &rescode, NULL, resbody)){
@@ -887,9 +886,9 @@ CBMAP *est_node_etch_doc_by_uri(ESTNODE *node, const char *uri){
     }
   }
   cblistpush(reqheads, "Content-Type: " ESTFORMTYPE, -1);
-  reqbody = cbdatumopen("", 0);
+  reqbody = cbdatumopen(NULL, -1);
   est_datum_printf(reqbody, "uri=%?", uri);
-  resbody = cbdatumopen("", 0);
+  resbody = cbdatumopen(NULL, -1);
   if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
                       reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
                       &rescode, NULL, resbody)){
@@ -940,9 +939,9 @@ int est_node_uri_to_id(ESTNODE *node, const char *uri){
     }
   }
   cblistpush(reqheads, "Content-Type: " ESTFORMTYPE, -1);
-  reqbody = cbdatumopen("", 0);
+  reqbody = cbdatumopen(NULL, -1);
   est_datum_printf(reqbody, "uri=%?", uri);
-  resbody = cbdatumopen("", 0);
+  resbody = cbdatumopen(NULL, -1);
   if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
                       reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
                       &rescode, NULL, resbody)){
@@ -1026,7 +1025,7 @@ double est_node_cache_usage(ESTNODE *node){
       free(vbuf);
     }
   }
-  resbody = cbdatumopen("", 0);
+  resbody = cbdatumopen(NULL, -1);
   if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
                       reqheads, NULL, -1, &rescode, NULL, resbody)){
     node->status = -1;
@@ -1093,7 +1092,7 @@ ESTNODERES *est_node_search(ESTNODE *node, ESTCOND *cond, int depth){
     }
   }
   cblistpush(reqheads, "Content-Type: " ESTFORMTYPE, -1);
-  reqbody = cbdatumopen("", 0);
+  reqbody = cbdatumopen(NULL, -1);
   if((phrase = est_cond_phrase(cond)) != NULL) est_datum_printf(reqbody, "phrase=%?", phrase);
   if((attrs = est_cond_attrs(cond)) != NULL){
     for(i = 0; i < cblistnum(attrs); i++){
@@ -1122,7 +1121,7 @@ ESTNODERES *est_node_search(ESTNODE *node, ESTCOND *cond, int depth){
   est_datum_printf(reqbody, "&skip=%d", est_cond_skip(cond));
   est_datum_printf(reqbody, "&mask=%d", est_cond_mask(cond));
   resheads = cbmapopenex(ESTMINIBNUM);
-  resbody = cbdatumopen("", 0);
+  resbody = cbdatumopen(NULL, -1);
   if(!est_url_shuttle(buf, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
                       reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
                       &rescode, resheads, resbody)){
@@ -1223,9 +1222,9 @@ int est_node_set_user(ESTNODE *node, const char *name, int mode){
     }
   }
   cblistpush(reqheads, "Content-Type: " ESTFORMTYPE, -1);
-  reqbody = cbdatumopen("", 0);
+  reqbody = cbdatumopen(NULL, -1);
   est_datum_printf(reqbody, "name=%?&mode=%d", name, mode);
-  resbody = cbdatumopen("", 0);
+  resbody = cbdatumopen(NULL, -1);
   if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
                       reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
                       &rescode, NULL, resbody)){
@@ -1263,13 +1262,13 @@ int est_node_set_link(ESTNODE *node, const char *url, const char *label, int cre
     }
   }
   cblistpush(reqheads, "Content-Type: " ESTFORMTYPE, -1);
-  reqbody = cbdatumopen("", 0);
+  reqbody = cbdatumopen(NULL, -1);
   if(credit >= 0){
     est_datum_printf(reqbody, "url=%?&label=%?&credit=%d", url, label, credit);
   } else {
     est_datum_printf(reqbody, "url=%?&label=%?", url, label);
   }
-  resbody = cbdatumopen("", 0);
+  resbody = cbdatumopen(NULL, -1);
   if(!est_url_shuttle(myurl, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
                       reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
                       &rescode, NULL, resbody)){
@@ -1928,7 +1927,7 @@ static void *est_url_shuttle_impl(void *targ){
   sp = cbmalloc(sizeof(int));
   *sp = sock;
   pthread_cleanup_push((void (*)(void *))est_sockpt_down, sp);
-  datum = cbdatumopen("", 0);
+  datum = cbdatumopen(NULL, -1);
   pthread_cleanup_push((void (*)(void *))cbdatumclose, datum);
   if(pxhost){
     est_datum_printf(datum, "%s %s HTTP/1.0\r\n", reqbody ? "POST" : "GET", url);
@@ -2062,7 +2061,7 @@ static int est_node_set_info(ESTNODE *node){
     cblistclose(node->links);
     node->links = NULL;
   }
-  resbody = cbdatumopen("", 0);
+  resbody = cbdatumopen(NULL, -1);
   if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
                       reqheads, NULL, -1, &rescode, NULL, resbody)){
     node->status = -1;
@@ -2186,10 +2185,10 @@ static int est_node_read_list(ESTNODE *node){
     }
   }
   cblistpush(reqheads, "Content-Type: " ESTFORMTYPE, -1);
-  reqbody = cbdatumopen("", 0);
+  reqbody = cbdatumopen(NULL, -1);
   est_datum_printf(reqbody, "max=%d", ESTITERUNIT);
   if(node->iprev) est_datum_printf(reqbody, "&prev=%?", node->iprev);
-  resbody = cbdatumopen("", 0);
+  resbody = cbdatumopen(NULL, -1);
   if(!est_url_shuttle(url, node->pxhost, node->pxport, node->timeout, ESTRESLIMSIZE, node->auth,
                       reqheads, cbdatumptr(reqbody), cbdatumsize(reqbody),
                       &rescode, NULL, resbody)){
