@@ -440,8 +440,9 @@ enum {                                   /* enumeration for open modes */
   ESTDBLCKNB = 1 << 5,                   /* lock without blocking */
   ESTDBPERFNG = 1 << 10,                 /* use perfect N-gram analyzer */
   ESTDBCHRCAT = 1 << 11,                 /* use character category analyzer */
-  ESTDBLARGE = 1 << 20,                  /* large tuning */
-  ESTDBHUGE = 1 << 21,                   /* huge tuning */
+  ESTDBSMALL = 1 << 20,                  /* small tuning */
+  ESTDBLARGE = 1 << 21,                  /* large tuning */
+  ESTDBHUGE = 1 << 22,                   /* huge tuning */
   ESTDBSCVOID = 1 << 25,                 /* store scores as void */
   ESTDBSCINT = 1 << 26,                  /* store scores as integer */
   ESTDBSCASIS = 1 << 27                  /* refrain from adjustment of scores */
@@ -495,6 +496,7 @@ const char *est_err_msg(int ecode);
    the application is responsible for exclusion control.  `ESTDBCREAT' can be added to by bitwise
    or: `ESTDBPERFNG', which means N-gram analysis is performed against European text also,
    `ESTDBCHACAT', which means character category analysis is performed instead of N-gram analysis,
+   `ESTDBSMALL', which means the index is tuned to register less than 50000 documents,
    `ESTDBLARGE', which means the index is tuned to register more than 300000 documents,
    `ESTDBHUGE', which means the index is tuned to register more than 1000000 documents,
    `ESTDBSCVOID', which means scores are stored as void, `ESTDBSCINT', which means scores are
@@ -712,8 +714,8 @@ void est_db_set_cache_size(ESTDB *db, size_t size, int anum, int tnum, int rnum)
  *************************************************************************************************/
 
 
-#define _EST_VERSION   "1.3.3"
-#define _EST_LIBVER    818
+#define _EST_VERSION   "1.3.6"
+#define _EST_LIBVER    821
 #define _EST_PROTVER   "1.0"
 
 #define _EST_PROJURL   "http://hyperestraier.sourceforge.net/"
