@@ -57,7 +57,7 @@ int g_sniphwidth = -1;                   /* width of beginning of the text */
 int g_snipawidth = -1;                   /* width around each highlighted word */
 int g_condgstep = -1;                    /* step of N-gram */
 int g_dotfidf = FALSE;                   /* whether to do TF-IDF tuning */
-int g_scancheck = FALSE;                 /* whether to check documents by scanning */
+int g_scancheck = -1;                    /* number of checked documents by scanning */
 int g_smplphrase = FALSE;                /* whether to use simplified phrase */
 int g_candetail = FALSE;                 /* whether to show detail link */
 int g_auxmin = -1;                       /* minimum hits to adopt the auxiliary index */
@@ -208,7 +208,7 @@ static int realmain(int argc, char **argv){
     } else if(cbstrfwimatch(rp, "dotfidf:")){
       if(!cbstricmp(skiplabel(rp), "true")) g_dotfidf = TRUE;
     } else if(cbstrfwimatch(rp, "scancheck:")){
-      if(!cbstricmp(skiplabel(rp), "true")) g_scancheck = TRUE;
+      g_scancheck = atoi(skiplabel(rp));
     } else if(cbstrfwimatch(rp, "smplphrase:")){
       if(!cbstricmp(skiplabel(rp), "true")) g_smplphrase = TRUE;
     } else if(cbstrfwimatch(rp, "candetail:")){
@@ -244,6 +244,7 @@ static int realmain(int argc, char **argv){
   if(g_sniphwidth < 0) showerror("sniphwidth is undefined.");
   if(g_snipawidth < 0) showerror("snipawidth is undefined.");
   if(g_condgstep < 1) showerror("condgstep is undefined.");
+  if(g_scancheck < 0) showerror("scancheck is undefined.");
   if(!g_smlrtune) showerror("smlrtune is undefined.");
   if(!g_spcache) showerror("spcache is undefined.");
   if(g_wildmax < 0) showerror("wildmax is undefined.");
@@ -272,7 +273,7 @@ static int realmain(int argc, char **argv){
     p_attr = "";
   }
   if(cbstrfwmatch(p_attr, "scan=")){
-    g_scancheck = !cbstricmp(p_attr + 5, "true");
+    g_scancheck = atoi(p_attr + 5);
     p_attr = "";
   }
   if(!(p_order = cbmapget(params, "order", -1, NULL))) p_order = "";
@@ -328,6 +329,7 @@ static int realmain(int argc, char **argv){
     if(g_spcache[0] != '\0') est_db_set_special_cache(g_db, g_spcache, SPCACHEMNUM);
     est_db_set_wildmax(g_db, g_wildmax);
   }
+  if(p_phrase[0] == '\0' || p_detail > 0) g_scancheck = 0;
   setsimilarphrase();
   /* show the page */
   showpage();
@@ -522,6 +524,7 @@ static void setsimilarphrase(void){
   char *ptr;
   int ksiz, vsiz;
   if(!cbstrfwimatch(p_phrase, ESTOPSIMILAR) && p_similar < 1) return;
+  g_scancheck = 0;
   if(g_smlrvnum < 1){
     p_phrase = "";
     return;
@@ -619,12 +622,14 @@ static void showpage(void){
     dnum = 0;
     miss = 0;
     for(i = 0; i < rnum; i++){
-      if(!(docs[dnum] = est_db_get_doc(g_db, res[i], dnum < p_pagenum * p_perpage || g_scancheck ?
-                                       0 : ESTGDNOATTR | ESTGDNOTEXT | ESTGDNOKWD))){
+      docs[dnum] = est_db_get_doc(g_db, res[i],
+                                  dnum < p_pagenum * p_perpage || g_scancheck > 0 ? 0 :
+                                  ESTGDNOATTR | ESTGDNOTEXT | ESTGDNOKWD);
+      if(!docs[dnum]){
         miss++;
         continue;
       }
-      if(g_scancheck && !est_db_scan_doc(g_db, docs[dnum], cond)){
+      if(g_scancheck > 0 && p_phrase[0] != '[' && !est_db_scan_doc(g_db, docs[dnum], cond)){
         est_doc_delete(docs[dnum]);
         miss++;
         continue;
@@ -634,6 +639,7 @@ static void showpage(void){
         est_doc_add_attr(docs[dnum], DATTRSCORE, numbuf);
       }
       dnum++;
+      g_scancheck--;
     }
     if((tnum <= MISSRETRYNUM && miss > 0 && max <= rnum && dnum < p_pagenum * p_perpage + 1) ||
        (p_pagenum == 1 && tnum == 0 && (hits < g_auxmin || hits < p_perpage) &&
