@@ -540,7 +540,7 @@ static int proccrawl(const char *rootdir, int mode){
   }
   cbmapclose(ulinks);
   kwords_reduce(waver->kwords, waver->seedkeynum, FALSE);
-  kwbuf = cbdatumopen("", 0);
+  kwbuf = cbdatumopen(NULL, -1);
   cbmapiterinit(waver->kwords);
   for(i = 0; (kbuf = cbmapiternext(waver->kwords, NULL)) != NULL; i++){
     if(i > 0) est_datum_printf(kwbuf, ", ");
@@ -718,7 +718,7 @@ static int procfetch(const char *url){
   const char *border, *vbuf;
   char *str;
   int i, code, vsiz;
-  raw = cbdatumopen("", 0);
+  raw = cbdatumopen(NULL, -1);
   heads = cbmapopen();
   links = cblistopen();
   doc = est_doc_new();
@@ -944,7 +944,7 @@ static void *geturldoc(void *args){
   }
   if(waver->interval > 0) est_usleep(waver->interval * 1000);
   alive = FALSE;
-  raw = cbdatumopen("", 0);
+  raw = cbdatumopen(NULL, -1);
   heads = cbmapopen();
   links = cblistopen();
   doc = est_doc_new();
@@ -1118,7 +1118,7 @@ static char *urltosavepath(const char *savedir, const char *url){
   const char *rp;
   int i;
   if((rp = strstr(url, "://")) != NULL) url = rp + 3;
-  buf = cbdatumopen("", 0);
+  buf = cbdatumopen(NULL, -1);
   elems = cbsplit(url, -1, "/");
   if(cbstrbwmatch(url, "/")) cblistpush(elems, "index.html", -1);
   est_datum_printf(buf, "%s", savedir);
