@@ -69,10 +69,12 @@ public class Database {
   public static final int DBPERFNG = 1 << 10;
   /** open mode: use character category analyzer */
   public static final int DBCHRCAT = 1 << 11;
+  /** open mode: small tuning */
+  public static final int DBSMALL = 1 << 20;
   /** open mode: large tuning */
-  public static final int DBLARGE = 1 << 20;
+  public static final int DBLARGE = 1 << 21;
   /** open mode: huge tuning */
-  public static final int DBHUGE = 1 << 21;
+  public static final int DBHUGE = 1 << 22;
   /** open mode: store scores as void */
   public static final int DBSCVOID = 1 << 25;
   /** open mode: store scores as integer */
@@ -168,7 +170,8 @@ public class Database {
    * for exclusion control.  `Database.DBCREAT' can be added to by bitwise or:
    * `Database.DBPERFNG', which means N-gram analysis is performed against European text also,
    * `Database.DBCHACAT', which means character category analysis is performed instead of N-gram
-   * analysis, `Database.DBLARGE', which means the index is tuned to register more than 300000
+   * analysis, `Database.DBSMALL', which means the index is tuned to register less than 50000
+   * documents, `Database.DBLARGE', which means the index is tuned to register more than 300000
    * documents, `Database.DBHUGE', which means the index is tuned to register more than 1000000
    * documents, `Database.DBSCVOID', which means scores are stored as void, `Database.DBSCINT',
    * which means scores are stored as 32-bit integer, `Database.DBSCASIS', which means scores
