@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Java binding of Hyper Estraier
- *                                                      Copyright (C) 2004-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2004-2006 Mikio Hirabayashi
  * This file is part of Hyper Estraier.
  * Hyper Estraier is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -18,8 +18,13 @@
 #include "myconf.h"
 
 
+/* global variables */
+JNIEnv *infoenv = NULL;
+
+
 /* private function prototypes */
 static void setecode(JNIEnv *env, jobject obj, int ecode);
+static void dbinform(const char *message, void *opaque);
 
 
 
@@ -28,6 +33,76 @@ static void setecode(JNIEnv *env, jobject obj, int ecode);
  *************************************************************************************************/
 
 
+/* search_meta */
+JNIEXPORT jobject JNICALL
+Java_estraier_Database_search_1meta(JNIEnv *env, jclass cls, jobjectArray dbary, jobject cond){
+  jfieldID fid;
+  jmethodID mid;
+  jobject dbobj, hintsobj, resobj;
+  jlong coreptr, condptr;
+  jintArray resary, idxary;
+  jint *resaryptr, *idxaryptr;
+  jboolean icresary, icidxary;
+  ESTMTDB **dbs;
+  CBMAP *hints;
+  int i, dbnum, *res, rnum;
+  fid = (*env)->GetFieldID(env, cls, "coreptr", "J");
+  if(!dbary || !cond || !isinstanceof(env, cond, CLSCOND)){
+    throwillarg(env);
+    return NULL;
+  }
+  dbnum = (int)((*env)->GetArrayLength(env, dbary));
+  dbs = cbmalloc(dbnum * sizeof(ESTMTDB) + 1);
+  for(i = 0; i < dbnum; i++){
+    dbobj = (*env)->GetObjectArrayElement(env, dbary, i);
+    if(!dbobj || !isinstanceof(env, dbobj, CLSDB)){
+      throwillarg(env);
+      return NULL;
+    }
+    coreptr = (*env)->GetLongField(env, dbobj, fid);
+    if(coreptr == 0){
+      throwillarg(env);
+      return NULL;
+    }
+    dbs[i] = (ESTMTDB *)(PTRNUM)coreptr;
+  }
+  cls = (*env)->GetObjectClass(env, cond);
+  fid = (*env)->GetFieldID(env, cls, "coreptr", "J");
+  condptr = (*env)->GetLongField(env, cond, fid);
+  hints = cbmapopen();
+  res = est_mtdb_search_meta(dbs, dbnum, (ESTCOND *)(PTRNUM)condptr, &rnum, hints);
+  if(!(resary = (*env)->NewIntArray(env, rnum / 2))){
+    throwoutmem(env);
+    return NULL;
+  }
+  resaryptr = (*env)->GetIntArrayElements(env, resary, &icresary);
+  for(i = 0; i < rnum; i += 2){
+    resaryptr[i/2] = res[i+1];
+  }
+  if(icresary == JNI_TRUE) (*env)->ReleaseIntArrayElements(env, resary, resaryptr, 0);
+  if(!(idxary = (*env)->NewIntArray(env, rnum / 2))){
+    throwoutmem(env);
+    return NULL;
+  }
+  idxaryptr = (*env)->GetIntArrayElements(env, idxary, &icidxary);
+  for(i = 0; i < rnum; i += 2){
+    idxaryptr[i/2] = res[i];
+  }
+  if(icidxary == JNI_TRUE) (*env)->ReleaseIntArrayElements(env, idxary, idxaryptr, 0);
+  cls = (*env)->FindClass(env, CLSRES);
+  mid = (*env)->GetMethodID(env, cls, "<init>", "([I[IL" CLSMAP ";J)V");
+  hintsobj = cbmaptoobj(env, hints);
+  if(!(resobj = (*env)->NewObject(env, cls, mid, resary, idxary, hintsobj,
+                                  (PTRNUM)est_cond_dup((ESTCOND *)(PTRNUM)condptr)))){
+    throwoutmem(env);
+    return NULL;
+  }
+  free(res);
+  cbmapclose(hints);
+  return resobj;
+}
+
+
 /* version */
 JNIEXPORT jstring JNICALL
 Java_estraier_Database_version(JNIEnv *env, jclass cls){
@@ -91,6 +166,7 @@ Java_estraier_Database_close(JNIEnv *env, jobject obj){
   jclass cls;
   jfieldID fid;
   jlong coreptr;
+  jobject infoobj;
   int err, ecode;
   cls = (*env)->GetObjectClass(env, obj);
   fid = (*env)->GetFieldID(env, cls, "coreptr", "J");
@@ -105,6 +181,10 @@ Java_estraier_Database_close(JNIEnv *env, jobject obj){
     err = TRUE;
   }
   (*env)->SetLongField(env, obj, fid, 0);
+  fid = (*env)->GetFieldID(env, cls, "informer", "L" CLSDBINFO ";");
+  if((infoobj = (*env)->GetObjectField(env, obj, fid)) != NULL)
+    (*env)->DeleteGlobalRef(env, infoobj);
+  (*env)->SetObjectField(env, obj, fid, NULL);
   return err ? FALSE : TRUE;
 }
 
@@ -134,6 +214,35 @@ Java_estraier_Database_fatal(JNIEnv *env, jobject obj){
 }
 
 
+/* add_attr_index */
+JNIEXPORT jboolean JNICALL
+Java_estraier_Database_add_1attr_1index(JNIEnv *env, jobject obj, jstring name, jint type){
+  jclass cls;
+  jfieldID fid;
+  jlong coreptr;
+  jboolean icname;
+  const char *tname;
+  cls = (*env)->GetObjectClass(env, obj);
+  fid = (*env)->GetFieldID(env, cls, "coreptr", "J");
+  coreptr = (*env)->GetLongField(env, obj, fid);
+  if(coreptr == 0 || !name || !isinstanceof(env, name, CLSSTRING)){
+    throwillarg(env);
+    return -1;
+  }
+  if(!(tname = (*env)->GetStringUTFChars(env, name, &icname))){
+    throwoutmem(env);
+    return -1;
+  }
+  if(!est_mtdb_add_attr_index((ESTMTDB *)(PTRNUM)coreptr, tname, type)){
+    setecode(env, obj, est_mtdb_error((ESTMTDB *)(PTRNUM)coreptr));
+    if(icname == JNI_TRUE) (*env)->ReleaseStringUTFChars(env, name, tname);
+    return FALSE;
+  }
+  if(icname == JNI_TRUE) (*env)->ReleaseStringUTFChars(env, name, tname);
+  return TRUE;
+}
+
+
 /* flush */
 JNIEXPORT jboolean JNICALL
 Java_estraier_Database_flush(JNIEnv *env, jobject obj, jint max){
@@ -197,6 +306,40 @@ Java_estraier_Database_optimize(JNIEnv *env, jobject obj, jint options){
 }
 
 
+/* merge */
+JNIEXPORT jboolean JNICALL
+Java_estraier_Database_merge(JNIEnv *env, jobject obj, jstring name, jint options){
+  jclass cls;
+  jfieldID fid;
+  jlong coreptr;
+  jboolean icname;
+  const char *tname;
+  int err;
+  cls = (*env)->GetObjectClass(env, obj);
+  fid = (*env)->GetFieldID(env, cls, "coreptr", "J");
+  coreptr = (*env)->GetLongField(env, obj, fid);
+  if(coreptr == 0){
+    throwillarg(env);
+    return FALSE;
+  }
+  if(!name || !isinstanceof(env, name, CLSSTRING)){
+    throwillarg(env);
+    return FALSE;
+  }
+  if(!(tname = (*env)->GetStringUTFChars(env, name, &icname))){
+    throwoutmem(env);
+    return FALSE;
+  }
+  err = FALSE;
+  if(!est_mtdb_merge((ESTMTDB *)(PTRNUM)coreptr, tname, options)){
+    setecode(env, obj, est_mtdb_error((ESTMTDB *)(PTRNUM)coreptr));
+    err = TRUE;
+  }
+  if(icname == JNI_TRUE) (*env)->ReleaseStringUTFChars(env, name, tname);
+  return err ? FALSE : TRUE;
+}
+
+
 /* put_doc */
 JNIEXPORT jboolean JNICALL
 Java_estraier_Database_put_1doc(JNIEnv *env, jobject obj, jobject doc, jint options){
@@ -471,9 +614,10 @@ Java_estraier_Database_search(JNIEnv *env, jobject obj, jobject cond){
   }
   if(icresary == JNI_TRUE) (*env)->ReleaseIntArrayElements(env, resary, resaryptr, 0);
   cls = (*env)->FindClass(env, CLSRES);
-  mid = (*env)->GetMethodID(env, cls, "<init>", "([IL" CLSMAP ";)V");
+  mid = (*env)->GetMethodID(env, cls, "<init>", "([IL" CLSMAP ";J)V");
   hintsobj = cbmaptoobj(env, hints);
-  if(!(resobj = (*env)->NewObject(env, cls, mid, resary, hintsobj))){
+  if(!(resobj = (*env)->NewObject(env, cls, mid, resary, hintsobj,
+                                  (PTRNUM)est_cond_dup((ESTCOND *)(PTRNUM)condptr)))){
     throwoutmem(env);
     return NULL;
   }
@@ -526,6 +670,51 @@ Java_estraier_Database_set_1cache_1size(JNIEnv *env, jobject obj,
 }
 
 
+/* set_wildmax */
+JNIEXPORT void JNICALL
+Java_estraier_Database_set_1wildmax(JNIEnv *env, jobject obj, jint num){
+  jclass cls;
+  jfieldID fid;
+  jlong coreptr;
+  cls = (*env)->GetObjectClass(env, obj);
+  fid = (*env)->GetFieldID(env, cls, "coreptr", "J");
+  coreptr = (*env)->GetLongField(env, obj, fid);
+  if(coreptr == 0){
+    throwillarg(env);
+    return;
+  }
+  est_mtdb_set_wildmax((ESTMTDB *)(PTRNUM)coreptr, num);
+}
+
+
+/* set_informer */
+JNIEXPORT void JNICALL
+Java_estraier_Database_set_1informer(JNIEnv *env, jobject obj, jobject informer){
+  jclass cls;
+  jfieldID fid;
+  jlong coreptr;
+  jobject oldobj;
+  cls = (*env)->GetObjectClass(env, obj);
+  fid = (*env)->GetFieldID(env, cls, "coreptr", "J");
+  coreptr = (*env)->GetLongField(env, obj, fid);
+  if(coreptr == 0){
+    throwillarg(env);
+    return;
+  }
+  fid = (*env)->GetFieldID(env, cls, "informer", "L" CLSDBINFO ";");
+  if((oldobj = (*env)->GetObjectField(env, obj, fid)) != NULL)
+    (*env)->DeleteGlobalRef(env, oldobj);
+  (*env)->SetObjectField(env, obj, fid, NULL);
+  if(!(informer = (*env)->NewGlobalRef(env, informer))){
+    throwoutmem(env);
+    return;
+  }
+  (*env)->SetObjectField(env, obj, fid, informer);
+  infoenv = env;
+  est_mtdb_set_informer((ESTMTDB *)(PTRNUM)coreptr, dbinform, informer);
+}
+
+
 
 /*************************************************************************************************
  * public objects
@@ -542,5 +731,19 @@ static void setecode(JNIEnv *env, jobject obj, int ecode){
 }
 
 
+/* callback function for database events */
+static void dbinform(const char *message, void *opaque){
+  jstring msgobj;
+  jobject opobj;
+  jclass cls;
+  jmethodID mid;
+  if(!(msgobj = (*infoenv)->NewStringUTF(infoenv, message))) return;
+  opobj = (jobject)opaque;
+  cls = (*infoenv)->GetObjectClass(infoenv, opobj);
+  mid = (*infoenv)->GetMethodID(infoenv, cls, "inform", "(L" CLSSTRING ";)V");
+  (*infoenv)->CallVoidMethod(infoenv, opobj, mid, msgobj);
+}
+
+
 
 /* END OF FILE */
