@@ -100,15 +100,14 @@ jobject cbmaptoobj(JNIEnv *env, CBMAP *map){
   jmethodID mid;
   jobject obj, kobj, vobj;
   const char *kbuf, *vbuf;
-  int ksiz;
   assert(list);
   cls = (*env)->FindClass(env, CLSLHMAP);
   mid = (*env)->GetMethodID(env, cls, "<init>", "()V");
   obj = (*env)->NewObject(env, cls, mid);
   mid = (*env)->GetMethodID(env, cls, "put", "(L" CLSOBJECT ";L" CLSOBJECT ";)L" CLSOBJECT ";");
   cbmapiterinit(map);
-  while((kbuf = cbmapiternext(map, &ksiz)) != NULL){
-    vbuf = cbmapget(map, kbuf, ksiz, NULL);
+  while((kbuf = cbmapiternext(map, NULL)) != NULL){
+    vbuf = cbmapiterval(kbuf, NULL);
     kobj = (*env)->NewStringUTF(env, kbuf);
     vobj = (*env)->NewStringUTF(env, vbuf);
     (*env)->CallObjectMethod(env, obj, mid, kobj, vobj);
