@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Pure Java interface of Hyper Estraier
- *                                                      Copyright (C) 2004-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2004-2006 Mikio Hirabayashi
  *                                                                           All rights reserved.
  * This file is part of Hyper Estraier.
  * Redistribution and use in source and binary forms, with or without modification, are
@@ -77,13 +77,16 @@ public class Node {
         sb.append(max);
       } else {
         sb.append("&max=");
-        sb.append(Integer.MAX_VALUE);
+        sb.append(Integer.MAX_VALUE / 2);
       }
       int options = cond.options();
       if(options > 0){
         sb.append("&options=");
         sb.append(options);
       }
+      int auxiliary = cond.auxiliary();
+      sb.append("&auxiliary=");
+      sb.append(auxiliary);
       if(depth > 0){
         sb.append("&depth=");
         sb.append(depth);
@@ -94,6 +97,10 @@ public class Node {
       sb.append(hwidth);
       sb.append("&awidth=");
       sb.append(awidth);
+      sb.append("&skip=");
+      sb.append(cond.skip());
+      sb.append("&mask=");
+      sb.append(cond.mask());
     } catch(UnsupportedEncodingException e){
       throw new RuntimeException(e);
     }
@@ -112,6 +119,9 @@ public class Node {
   private int dnum;
   private int wnum;
   private double size;
+  private List admins;
+  private List users;
+  private List links;
   private int wwidth;
   private int hwidth;
   private int awidth;
@@ -133,6 +143,9 @@ public class Node {
     dnum = -1;
     wnum = -1;
     size = -1.0;
+    admins = null;
+    users = null;
+    links = null;
     wwidth = 480;
     hwidth = 96;
     awidth = 96;
@@ -179,6 +192,54 @@ public class Node {
   public int status(){
     return status;
   }
+  /**
+   * Synchronize updating contents of the database.
+   * @return true if success, else it is false.
+   */
+  public boolean sync(){
+    status = -1;
+    if(url == null) return false;
+    try {
+      URL purl = new URL(url);
+      URL eurl = new URL("http://" + purl.getHost() + ":" + purl.getPort() +
+                         purl.getPath() + "/sync");
+      List reqheads = new ArrayList(2);
+      if(auth != null)
+        reqheads.add("Authorization: Basic " + Utility.base_encode(auth.getBytes()));
+      reqheads.add("Content-Type: application/x-www-form-urlencoded");
+      byte[] reqbody = new byte[0];
+      status = Utility.shuttle_url(eurl.toString(), pxhost, pxport, timeout,
+                                   reqheads, reqbody, null, null);
+      if(status != 200) return false;
+      return true;
+    } catch(Exception e){
+      return false;
+    }
+  }
+  /**
+   * Optimize the database.
+   * @return true if success, else it is false.
+   */
+  public boolean optimize(){
+    status = -1;
+    if(url == null) return false;
+    try {
+      URL purl = new URL(url);
+      URL eurl = new URL("http://" + purl.getHost() + ":" + purl.getPort() +
+                         purl.getPath() + "/optimize");
+      List reqheads = new ArrayList(2);
+      if(auth != null)
+        reqheads.add("Authorization: Basic " + Utility.base_encode(auth.getBytes()));
+      reqheads.add("Content-Type: application/x-www-form-urlencoded");
+      byte[] reqbody = new byte[0];
+      status = Utility.shuttle_url(eurl.toString(), pxhost, pxport, timeout,
+                                   reqheads, reqbody, null, null);
+      if(status != 200) return false;
+      return true;
+    } catch(Exception e){
+      return false;
+    }
+  }
   /**
    * Add a document.
    * @param doc a document object to register.
@@ -517,15 +578,63 @@ public class Node {
     return wnum;
   }
   /**
-   * Get the size of the datbase of a node.
-   * @return the size of the datbase of the node.  On error, -1.0 is returned.
+   * Get the size of the datbase.
+   * @return the size of the datbase.  On error, -1.0 is returned.
    */
   public double size(){
     if(size < 0.0) set_info();
     return size;
   }
   /**
-   * Search documents corresponding a condition.
+   * Get the usage ratio of the cache.
+   * @return the usage ratio of the cache.  On error, -1.0 is returned.
+   */
+  public double cache_usage(){
+    status = -1;
+    if(url == null) return -1.0;
+    try {
+      URL purl = new URL(url);
+      URL eurl = new URL("http://" + purl.getHost() + ":" + purl.getPort() +
+                         purl.getPath() + "/cacheusage");
+      List reqheads = new ArrayList(2);
+      if(auth != null)
+        reqheads.add("Authorization: Basic " + Utility.base_encode(auth.getBytes()));
+      ByteArrayOutputStream resbody = new ByteArrayOutputStream();
+      status = Utility.shuttle_url(eurl.toString(), pxhost, pxport, timeout,
+                                   reqheads, null, null, resbody);
+      if(status != 200) return -1.0;
+      return Double.parseDouble(resbody.toString("UTF-8").trim());
+    } catch(Exception e){
+      return -1.0;
+    }
+  }
+  /**
+   * Get a list of names of administrators.
+   * @return a list object of names of administrators.  On error, `null' is returned.
+   */
+  public List admins(){
+    if(admins == null) set_info();
+    return admins;
+  }
+  /**
+   * Get a list of names of users.
+   * @return a list object of names of users.  On error, `null' is returned.
+   */
+  public List users(){
+    if(users == null) set_info();
+    return users;
+  }
+  /**
+   * Get a list of expressions of links.
+   * @return a list object of expressions of links.  Each element is a TSV string and has three
+   * fields of the URL, the label, and the score.  On error, `null' is returned.
+   */
+  public List links(){
+    if(links == null) set_info();
+    return links;
+  }
+  /**
+   * Search for documents corresponding a condition.
    * @param cond a condition object.
    * @param depth the depth of meta search.
    * @return a node result object.  On error, `null' is returned.
@@ -721,12 +830,41 @@ public class Node {
         wnum = -1;
         size = -1.0;
       }
+      if(lines.length < 2) return;
+      int lnum = 1;
+      if(lnum < lines.length && lines[lnum].length() < 1) lnum++;
+      admins = new ArrayList();
+      while(lnum < lines.length){
+        String line = lines[lnum];
+        if(line.length() < 1) break;
+        admins.add(line);
+        lnum++;
+      }
+      if(lnum < lines.length && lines[lnum].length() < 1) lnum++;
+      users = new ArrayList();
+      while(lnum < lines.length){
+        String line = lines[lnum];
+        if(line.length() < 1) break;
+        users.add(line);
+        lnum++;
+      }
+      if(lnum < lines.length && lines[lnum].length() < 1) lnum++;
+      links = new ArrayList();
+      while(lnum < lines.length){
+        String line = lines[lnum];
+        if(line.length() < 1) break;
+        if(Utility.split_fields(line).length == 3) links.add(line);
+        lnum++;
+      }
     } catch(Exception e){
       name = null;
       label = null;
       dnum = -1;
       wnum = -1;
       size = -1.0;
+      admins = null;
+      users = null;
+      links = null;
     }
   }
 }
