@@ -266,7 +266,7 @@ int master_init(const char *rootdir){
     fprintf(ofp, "snipawidth: 96\n");
     fprintf(ofp, "\n");
     fprintf(ofp, "# whether to check documents by scanning (0:no, 1:yes)\n");
-    fprintf(ofp, "scancheck: 0\n");
+    fprintf(ofp, "scancheck: 1\n");
     fprintf(ofp, "\n");
     fprintf(ofp, "# number of keywords for similarity search (0 means disabled)\n");
     fprintf(ofp, "smlrvnum: 32\n");
@@ -642,7 +642,7 @@ int nmgr_sync(NMGR *nmgr, int phis){
     node = (NODE *)vbuf;
     est_mtdb_add_meta(node->db, NMKNAME, node->name);
     est_mtdb_add_meta(node->db, NMKLABEL, node->label);
-    datum = cbdatumopen("", 0);
+    datum = cbdatumopen(NULL, -1);
     cbmapiterinit(node->admins);
     while((kbuf = cbmapiternext(node->admins, &ksiz)) != NULL){
       cbdatumcat(datum, kbuf, ksiz);
@@ -650,7 +650,7 @@ int nmgr_sync(NMGR *nmgr, int phis){
     }
     est_mtdb_add_meta(node->db, NMKADMINS, cbdatumptr(datum));
     cbdatumclose(datum);
-    datum = cbdatumopen("", 0);
+    datum = cbdatumopen(NULL, -1);
     cbmapiterinit(node->users);
     while((kbuf = cbmapiternext(node->users, &ksiz)) != NULL){
       cbdatumcat(datum, kbuf, ksiz);
@@ -658,7 +658,7 @@ int nmgr_sync(NMGR *nmgr, int phis){
     }
     est_mtdb_add_meta(node->db, NMKUSERS, cbdatumptr(datum));
     cbdatumclose(datum);
-    datum = cbdatumopen("", 0);
+    datum = cbdatumopen(NULL, -1);
     cbmapiterinit(node->links);
     while((kbuf = cbmapiternext(node->links, &ksiz)) != NULL){
       vbuf = cbmapget(node->links, kbuf, ksiz, &vsiz);
