@@ -37,6 +37,9 @@
 #define NODENAMEMAX    128               /* maximum length of node name  */
 #define MINIBNUM       31                /* bucket number of a small map */
 #define KWORDNUM       32                /* number of shown keywords */
+#define CONDATTRMAX    9                 /* maximum number of attribute conditions */
+#define DEFMAXSRCH     10                /* default max number of retrieved documents */
+#define SELFCREDIT     10000             /* credit of the node itself */
 
 #define CONFFILE       "_conf"           /* name of the configuration file */
 #define USERFILE       "_user"           /* name of the user list file */
@@ -48,6 +51,7 @@
 #define NODEDIR        "_node"           /* name of the node directory */
 #define SESSDIR        "_sess"           /* name of the session directory */
 #define NULLDEV        "/dev/null"       /* path of null device */
+#define SEARCHHELPER   "estbutler search"  /* command name of the search helper */
 
 #define MMKMAGIC       "magic"           /* meta key of the magic number of meta DB */
 #define MMKMAGVAL      "[ESTMASTER]"     /* value of the magic number of meta DB */
@@ -57,6 +61,11 @@
 #define NMKUSERS       "users"           /* meta key of the users of node DB */
 #define NMKLINKS       "links"           /* meta key of the links of node DB */
 
+#define DATTRNDURL     "#nodeurl"        /* name of the pseudo-attribute of the node URL */
+#define DATTRNDLABEL   "#nodelabel"      /* name of the pseudo-attribute of the node label */
+#define DATTRNDSCORE   "#nodescore"      /* name of the pseudo-attribute of the node score */
+#define DATTRLFILE     "_lfile"          /* name of the attribute of the file name */
+
 enum {                                   /* enumeration for running modes */
   RM_NORMAL = 1,                         /* normal */
   RM_RDONLY = 2                          /* read only */
