@@ -331,10 +331,12 @@ module Estraier
     DBPERFNG = 1 << 10
     # open mode: use character category analyzer
     DBCHRCAT = 1 << 11
+    # open mode: small tuning
+    DBSMALL = 1 << 20
     # open mode: large tuning
-    DBLARGE = 1 << 20
+    DBLARGE = 1 << 21
     # open mode: huge tuning
-    DBHUGE = 1 << 21
+    DBHUGE = 1 << 22
     # open mode: store scores as void
     DBSCVOID = 1 << 25
     # open mode: store scores as integer
@@ -398,7 +400,8 @@ module Estraier
     # `Database::DBNOLCK' is used, the application is responsible for exclusion control.
     # `Database::DBCREAT' can be added to by bitwise or: `Database::DBPERFNG', which means N-gram
     # analysis is performed against European text also, `Database::DBCHACAT', which means
-    # character category analysis is performed instead of N-gram analysis, `Database::DBLARGE',
+    # character category analysis is performed instead of N-gram analysis, `Database::DBSMALL',
+    # which means the index is tuned to register less than 50000 documents, `Database::DBLARGE',
     # which means the index is tuned to register more than 300000 documents, `Database::DBHUGE',
     # which means the index is tuned to register more than 1000000 documents,
     # `Database::DBSCVOID', which means scores are stored as void, `Database::DBSCINT', which
