@@ -665,6 +665,7 @@ static void db_init(void){
   rb_define_const(cls_db, "DBLCKNB", INT2FIX(ESTDBLCKNB));
   rb_define_const(cls_db, "DBPERFNG", INT2FIX(ESTDBPERFNG));
   rb_define_const(cls_db, "DBCHRCAT", INT2FIX(ESTDBCHRCAT));
+  rb_define_const(cls_db, "DBSMALL", INT2FIX(ESTDBSMALL));
   rb_define_const(cls_db, "DBLARGE", INT2FIX(ESTDBLARGE));
   rb_define_const(cls_db, "DBHUGE", INT2FIX(ESTDBHUGE));
   rb_define_const(cls_db, "DBSCVOID", INT2FIX(ESTDBSCVOID));
@@ -1192,7 +1193,7 @@ static VALUE cbmaptoobj(CBMAP *map){
   obj = rb_hash_new();
   cbmapiterinit(map);
   while((kbuf = cbmapiternext(map, &ksiz)) != NULL){
-    vbuf = cbmapget(map, kbuf, ksiz, &vsiz);
+    vbuf = cbmapiterval(kbuf, &vsiz);
     rb_hash_aset(obj, rb_str_new(kbuf, ksiz), rb_str_new(vbuf, vsiz));
   }
   return obj;
