@@ -1,7 +1,7 @@
-#! /usr/local/bin/ruby -w
+#! /usr/bin/ruby -w
 #--
 # Ruby interface of Hyper Estraier
-#                                                       Copyright (C) 2004-2005 Mikio Hirabayashi
+#                                                       Copyright (C) 2004-2006 Mikio Hirabayashi
 #                                                                            All rights reserved.
 #  This file is part of Hyper Estraier.
 #  Redistribution and use in source and binary forms, with or without modification, are
@@ -63,6 +63,10 @@ def main(args)
     rv = runuriid(args)
   when "inform"
     rv = runinform(args)
+  when "sync"
+    rv = runsync(args)
+  when "optimize"
+    rv = runoptimize(args)
   when "search"
     rv = runsearch(args)
   when "setuser"
@@ -78,23 +82,28 @@ end
 
 # print usage and exit
 def usage()
-  printf("%s: command line utility for the node API of Hyper Estraier\n", $0)
-  printf("\n")
-  printf("usage:\n")
-  printf("  %s put [-proxy host port] [-tout num] [-auth user pass] nurl [file]\n", $0)
-  printf("  %s out [-proxy host port] [-tout num] [-auth user pass] nurl expr\n", $0)
-  printf("  %s edit [-proxy host port] [-tout num] [-auth user pass]" \
-         " nurl expr name [value]\n", $0)
-  printf("  %s get [-proxy host port] [-tout num] [-auth user pass] nurl expr [attr]\n", $0)
-  printf("  %s etch [-proxy host port] [-tout num] [-auth user pass] nurl expr\n", $0)
-  printf("  %s uriid [-proxy host port] [-tout num] [-auth user pass] nurl uri\n", $0)
-  printf("  %s inform [-proxy host port] [-tout num] [-auth user pass] nurl\n", $0)
-  printf("  %s search [-proxy host port] [-tout num] [-auth user pass] [-kw] [-sf]" \
-         " [-attr expr] [-ord expr] [-max num] [-dpt num] nurl [phrase]\n", $0)
-  printf("  %s setuser [-proxy host port] [-tout num] [-auth user pass] nurl name mode\n", $0)
-  printf("  %s setlink [-proxy host port] [-tout num] [-auth user pass]" \
-         " nurl url label credit\n", $0)
-  printf("\n")
+  STDERR.printf("%s: command line utility for the node API of Hyper Estraier\n", $0)
+  STDERR.printf("\n")
+  STDERR.printf("usage:\n")
+  STDERR.printf("  %s put [-proxy host port] [-tout num] [-auth user pass] nurl [file]\n", $0)
+  STDERR.printf("  %s out [-proxy host port] [-tout num] [-auth user pass] nurl expr\n", $0)
+  STDERR.printf("  %s edit [-proxy host port] [-tout num] [-auth user pass]" \
+                " nurl expr name [value]\n", $0)
+  STDERR.printf("  %s get [-proxy host port] [-tout num] [-auth user pass]" \
+                " nurl expr [attr]\n", $0)
+  STDERR.printf("  %s etch [-proxy host port] [-tout num] [-auth user pass] nurl expr\n", $0)
+  STDERR.printf("  %s uriid [-proxy host port] [-tout num] [-auth user pass] nurl uri\n", $0)
+  STDERR.printf("  %s inform [-proxy host port] [-tout num] [-auth user pass] nurl\n", $0)
+  STDERR.printf("  %s sync [-proxy host port] [-tout num] [-auth user pass] nurl\n", $0)
+  STDERR.printf("  %s optimize [-proxy host port] [-tout num] [-auth user pass] nurl\n", $0)
+  STDERR.printf("  %s search [-proxy host port] [-tout num] [-auth user pass] [-kw] [-sf]" \
+                " [-attr expr] [-ord expr] [-max num] [-sk num] [-dpt num] [-mask num]" \
+                " nurl [phrase]\n", $0)
+  STDERR.printf("  %s setuser [-proxy host port] [-tout num] [-auth user pass]" \
+                " nurl name mode\n", $0)
+  STDERR.printf("  %s setlink [-proxy host port] [-tout num] [-auth user pass]" \
+                " nurl url label credit\n", $0)
+  STDERR.printf("\n")
   exit(1)
 end
 
@@ -371,6 +380,74 @@ def runinform(args)
 end
 
 
+# parse arguments of the sync command
+def runsync(args)
+  nurl = nil
+  i = 1
+  while i < args.length
+    if !nurl && args[i] =~ /^-/
+      if args[i] == "-proxy"
+        usage if (i += 1) >= args.length
+        $pxhost = args[i]
+        usage if (i += 1) >= args.length
+        $pxport = args[i].to_i
+      elsif args[i] == "-tout"
+        usage if (i += 1) >= args.length
+        $timeout = args[i].to_i
+      elsif args[i] == "-auth"
+        usage if (i += 1) >= args.length
+        $authname = args[i]
+        usage if (i += 1) >= args.length
+        $authpass = args[i]
+      else
+        usage
+      end
+    elsif !nurl
+      nurl = args[i]
+    else
+      usage
+    end
+    i += 1
+  end
+  usage if !nurl
+  procsync(nurl)
+end
+
+
+# parse arguments of the optimize command
+def runoptimize(args)
+  nurl = nil
+  i = 1
+  while i < args.length
+    if !nurl && args[i] =~ /^-/
+      if args[i] == "-proxy"
+        usage if (i += 1) >= args.length
+        $pxhost = args[i]
+        usage if (i += 1) >= args.length
+        $pxport = args[i].to_i
+      elsif args[i] == "-tout"
+        usage if (i += 1) >= args.length
+        $timeout = args[i].to_i
+      elsif args[i] == "-auth"
+        usage if (i += 1) >= args.length
+        $authname = args[i]
+        usage if (i += 1) >= args.length
+        $authpass = args[i]
+      else
+        usage
+      end
+    elsif !nurl
+      nurl = args[i]
+    else
+      usage
+    end
+    i += 1
+  end
+  usage if !nurl
+  procoptimize(nurl)
+end
+
+
 # parse arguments of the search command
 def runsearch(args)
   nurl = nil
@@ -378,8 +455,10 @@ def runsearch(args)
   attrs = []
   ord = nil
   max = SEARCHMAX
+  skip = 0
   opts = 0
   depth = 0
+  mask = 0
   i = 1
   while i < args.length
     if !nurl && args[i] =~ /^-/
@@ -409,9 +488,15 @@ def runsearch(args)
       elsif args[i] == "-max"
         usage if (i += 1) >= args.length
         max = args[i].to_i
+      elsif args[i] == "-sk"
+        usage if (i += 1) >= args.length
+        skip = args[i].to_i
       elsif args[i] == "-dpt"
         usage if (i += 1) >= args.length
         depth = args[i].to_i
+      elsif args[i] == "-mask"
+        usage if (i += 1) >= args.length
+        mask = args[i].to_i
       else
         usage
       end
@@ -425,7 +510,7 @@ def runsearch(args)
     i += 1
   end
   usage if !nurl || depth < 0
-  procsearch(nurl, phrase, attrs, ord, max, opts, depth)
+  procsearch(nurl, phrase, attrs, ord, max, skip, opts, depth, mask)
 end
 
 
@@ -660,17 +745,48 @@ def procinform(nurl)
   dnum = node.doc_num
   wnum = node.word_num
   size = node.size
-  if !name || !label || dnum < 0 || wnum < 0 || size < 0.0
+  ratio = node.cache_usage
+  if !name || !label || dnum < 0 || wnum < 0 || size < 0.0 || ratio < 0.0
+    printerror("failed: " + node.status.to_s)
+    return 1
+  end
+  printf("%s\t%s\t%d\t%d\t%.0f\t%.6f\n", name, label, dnum, wnum, size, ratio)
+  return 0
+end
+
+
+# perform the sync command
+def procsync(nurl)
+  node = Node::new
+  node.set_url(nurl)
+  node.set_proxy($pxhost, $pxport) if $pxhost
+  node.set_timeout($timeout) if $timeout > 0
+  node.set_auth($authname, $authpass) if $authname
+  if !node.sync
+    printerror("failed: " + node.status.to_s)
+    return 1
+  end
+  return 0
+end
+
+
+# perform the optimize command
+def procoptimize(nurl)
+  node = Node::new
+  node.set_url(nurl)
+  node.set_proxy($pxhost, $pxport) if $pxhost
+  node.set_timeout($timeout) if $timeout > 0
+  node.set_auth($authname, $authpass) if $authname
+  if !node.optimize
     printerror("failed: " + node.status.to_s)
     return 1
   end
-  printf("%s\t%s\t%d\t%d\t%.0f\n", name, label, dnum, wnum, size)
   return 0
 end
 
 
 # perform the search command
-def procsearch(nurl, phrase, attrs, ord, max, opts, depth)
+def procsearch(nurl, phrase, attrs, ord, max, skip, opts, depth, mask)
   node = Node::new
   node.set_url(nurl)
   node.set_proxy($pxhost, $pxport) if $pxhost
@@ -683,7 +799,9 @@ def procsearch(nurl, phrase, attrs, ord, max, opts, depth)
   end
   cond.set_order(ord) if ord
   cond.set_max(max) if max >= 0
+  cond.set_skip(skip) if skip >= 0
   cond.set_options(opts)
+  cond.set_mask(mask)
   nres = node.search(cond, depth)
   if !nres
     printerror("failed: " + node.status.to_s)
@@ -712,6 +830,17 @@ def procsearch(nurl, phrase, attrs, ord, max, opts, depth)
   printf("WORDNUM\t%s\n", value) if value
   value = nres.hint("TIME")
   printf("TIME\t%s\n", value) if value
+  value = nres.hint("TIME#i")
+  printf("TIME#i\t%s\n", value) if value
+  for i in 0...1024
+    key = "TIME#" + i.to_s
+    value = nres.hint(key)
+    if value
+      printf("%s\t%s\n", key, value)
+    else
+      break
+    end
+  end
   for i in 0...1024
     key = "LINK#" + i.to_s
     value = nres.hint(key)
@@ -731,8 +860,8 @@ def procsearch(nurl, phrase, attrs, ord, max, opts, depth)
     for j in 0...names.length
       printf("%s=%s\n", names[j], rdoc.attr(names[j]))
     end
-    vector = rdoc.vector
-    printf("%%VECTOR\t%s\n", vector) if vector.length > 0
+    keywords = rdoc.keywords
+    printf("%%VECTOR\t%s\n", keywords) if keywords.length > 0
     printf("\n")
     printf("%s", rdoc.snippet)
   end
