@@ -1,6 +1,6 @@
 #--
 # Ruby interface of Hyper Estraier
-#                                                       Copyright (C) 2004-2005 Mikio Hirabayashi
+#                                                       Copyright (C) 2004-2006 Mikio Hirabayashi
 #                                                                            All rights reserved.
 #  This file is part of Hyper Estraier.
 #  Redistribution and use in source and binary forms, with or without modification, are
@@ -53,9 +53,9 @@ module EstraierPure
     def add_attr(name, value)
       Utility::check_types({ name=>String, value=>String }) if $DEBUG
       name = name.gsub(/[ \t\r\n\v\f]+/, " ")
-      name = name.lstrip.rstrip.squeeze(" ")
+      name = name.strip.squeeze(" ")
       value = value.gsub(/[ \t\r\n\v\f]+/, " ")
-      value = value.lstrip.rstrip.squeeze(" ")
+      value = value.strip.squeeze(" ")
       @attrs[name] = value
       nil
     end
@@ -65,7 +65,7 @@ module EstraierPure
     def add_text(text)
       Utility::check_types({ text=>String }) if $DEBUG
       text = text.gsub(/[ \t\r\n\v\f]+/, " ")
-      text = text.lstrip.rstrip.squeeze(" ")
+      text = text.strip.squeeze(" ")
       @dtexts.push(text) if text.length
       nil
     end
@@ -75,18 +75,27 @@ module EstraierPure
     def add_hidden_text(text)
       Utility::check_types({ text=>String }) if $DEBUG
       text = text.gsub(/[ \t\r\n\v\f]+/, " ")
-      text = text.lstrip.rstrip.squeeze(" ")
+      text = text.strip.squeeze(" ")
       @htexts.push(text) if text.length
       nil
     end
+    # Attache keywords.
+    # `kwords' specifies a map object of keywords.  Keys of the map should be keywords of the
+    # document and values should be their scores in decimal string.
+    # The return value is always `nil'.
+    def set_keywords(kwords)
+      Utility::check_types({ kwords=>Hash }) if $DEBUG
+      @kwords = kwords
+      nil
+    end
     # Get the ID number.
     # The return value is the ID number of the document object.  If the object has never been
     # registered, -1 is returned.
     def id()
       @id
     end
-    # Get a list of attribute names of a document object.
-    # The return value is a list object of attribute names.
+    # Get an array of attribute names of a document object.
+    # The return value is an array object of attribute names.
     def attr_names()
       @attrs.keys.sort
     end
@@ -97,8 +106,8 @@ module EstraierPure
       Utility::check_types({ name=>String }) if $DEBUG
       @attrs[name]
     end
-    # Get a list of sentences of the text.
-    # The return value is a list object of sentences of the text.
+    # Get an array of sentences of the text.
+    # The return value is an array object of sentences of the text.
     def texts()
       @dtexts
     end
@@ -120,6 +129,13 @@ module EstraierPure
       for i in 0...keys.length
         buf.printf("%s=%s\n", keys[i], @attrs[keys[i]])
       end
+      if @kwords
+        buf.printf("%%VECTOR")
+        @kwords.each() do |key, value|
+          buf.printf("\t%s\t%s", key, value)
+        end
+        buf.printf("\n")
+      end
       buf.printf("\n")
       for i in 0...@dtexts.length
         buf.printf("%s\n", @dtexts[i])
@@ -129,6 +145,12 @@ module EstraierPure
       end
       buf.string
     end
+    # Get attached keywords.
+    # The return value is a map object of keywords and their scores in decimal string.  If no
+    # keyword is attached, `nil' is returned.
+    def keywords()
+      @kwords
+    end
     #--------------------------------
     # private methods
     #--------------------------------
@@ -141,16 +163,28 @@ module EstraierPure
       @attrs = {}
       @dtexts = []
       @htexts = []
+      @kwords = nil
       if draft.length
-        lines = draft.split(/\n/)
+        lines = draft.split(/\n/, -1)
         num = 0
         while num < lines.length
           line = lines[num]
-          line = line.gsub(/[ \t\r\n\v\f]+/, " ")
-          line = line.lstrip.rstrip.squeeze(" ")
           num += 1
           break if line.length < 1
-          next if line =~ /^%/
+          if line =~ /^%/
+            if line =~ /^%VECTOR\t/
+              @kwords = {} unless @kwords
+              fields = line.split(/\t/)
+              i = 1
+              while i < fields.length - 1
+                @kwords[fields[i]] = fields[i+1]
+                i += 2
+              end
+            end
+            next
+          end
+          line = line.gsub(/[ \t\r\n\v\f]+/, " ")
+          line = line.strip.squeeze(" ")
           if idx = line.index("=")
             key = line[0...idx]
             value = line[idx+1...line.length]
@@ -160,8 +194,8 @@ module EstraierPure
         while num < lines.length
           line = lines[num]
           next unless line.length
-          if line[0] == '\t'
-            @htexts.push(line)
+          if line[0] == 0x9
+            @htexts.push(line[1...line.length]) if line.length > 1
           else
             @dtexts.push(line)
           end
@@ -178,20 +212,18 @@ module EstraierPure
     # public constants
     #--------------------------------
     public
-    # option: check N-gram keys skipping by three
-    SURE = 1 << 0
-    # option: check N-gram keys skipping by two
-    USUAL = 1 << 1
-    # option: without TF-IDF tuning
-    FAST = 1 << 2
-    # option: with the simplified phrase
-    AGITO = 1 << 3
     # option: check every N-gram key
-    NOIDF = 1 << 4
+    SURE = 1 << 0
     # option: check N-gram keys skipping by one
+    USUAL = 1 << 1
+    # option: check N-gram keys skipping by two
+    FAST = 1 << 2
+    # option: check N-gram keys skipping by three
+    AGITO = 1 << 3
+    # option: without TF-IDF tuning
+    NOIDF = 1 << 4
+    # option: with the simplified phrase
     SIMPLE = 1 << 10
-    # option: attach keywords
-    ETCH = 1 << 20
     #--------------------------------
     # public methods
     #--------------------------------
@@ -202,7 +234,7 @@ module EstraierPure
     def set_phrase(phrase)
       Utility::check_types({ phrase=>String }) if $DEBUG
       phrase = phrase.gsub(/[ \t\r\n\v\f]+/, " ")
-      phrase = phrase.lstrip.rstrip.squeeze(" ")
+      phrase = phrase.strip.squeeze(" ")
       @phrase = phrase
       nil
     end
@@ -212,7 +244,7 @@ module EstraierPure
     def add_attr(expr)
       Utility::check_types({ expr=>String }) if $DEBUG
       expr = expr.gsub(/[ \t\r\n\v\f]+/, " ")
-      expr = expr.lstrip.rstrip.squeeze(" ")
+      expr = expr.strip.squeeze(" ")
       @attrs.push(expr)
       nil
     end
@@ -222,7 +254,7 @@ module EstraierPure
     def set_order(expr)
       Utility::check_types({ expr=>String }) if $DEBUG
       expr = expr.gsub(/[ \t\r\n\v\f]+/, " ")
-      expr = expr.lstrip.rstrip.squeeze(" ")
+      expr = expr.strip.squeeze(" ")
       @order = expr
       nil
     end
@@ -232,7 +264,15 @@ module EstraierPure
     # The return value is always `nil'.
     def set_max(max)
       Utility::check_types({ max=>Integer }) if $DEBUG
-      @max = max
+      @max = max if max >= 0
+      nil
+    end
+    # Set the number of skipped documents.
+    # `skip' specifies the number of documents to be skipped in the search result.
+    # The return value is always `nil'.
+    def set_skip(skip)
+      Utility::check_types({ skip=>Integer }) if $DEBUG
+      @skip = skip if skip >= 0
       nil
     end
     # Set options of retrieval.
@@ -240,15 +280,33 @@ module EstraierPure
     # key, `Condition::USU', which is the default, specifies that it checks N-gram keys
     # with skipping one key, `Condition::FAST' skips two keys, `Condition::AGITO'
     # skips three keys, `Condition::NOIDF' specifies not to perform TF-IDF tuning,
-    # `Condition::SIMPLE' specifies to use simplified phrase.  `Condition::ETCH' specifies to
-    # attach the keyword vector.  Each option can be specified at the same time by bitwise or.
-    # If keys are skipped, though search speed is improved, the relevance ratio grows less.
+    # `Condition::SIMPLE' specifies to use simplified phrase.  Each option can be specified at
+    # the same time by bitwise or.  If keys are skipped, though search speed is improved, the
+    # relevance ratio grows less.
     # The return value is always `nil'.
     def set_options(options)
       Utility::check_types({ options=>Integer }) if $DEBUG
       @options |= options
       nil
     end
+    # Set permission to adopt result of the auxiliary index.
+    # `min' specifies the minimum hits to adopt result of the auxiliary index.  If it is not more
+    # than 0, the auxiliary index is not used.  By default, it is 32.
+    # The return value is always `nil'.
+    def set_auxiliary(min)
+      Utility::check_types({ min=>Integer }) if $DEBUG
+      @auxiliary = min
+      nil
+    end
+    # Set the mask of targets of meta search.
+    # `mask' specifies a masking number.  1 means the first target, 2 means the second target, 4
+    # means the third target, and power values of 2 and their summation compose the mask.
+    # The return value is always `nil'.
+    def set_mask(mask)
+      Utility::check_types({ mask=>Integer }) if $DEBUG
+      @mask = mask
+      nil
+    end
     # Get the search phrase.
     # The return value is the search phrase.
     def phrase()
@@ -269,11 +327,26 @@ module EstraierPure
     def max()
       @max
     end
+    # Get the number of skipped documents.
+    # The return value is the number of documents to be skipped in the search result.
+    def skip()
+      @skip
+    end
     # Get options of retrieval.
     # The return value is options by bitwise or.
     def options()
       @options
     end
+    # Get permission to adopt result of the auxiliary index.
+    # The return value is permission to adopt result of the auxiliary index.
+    def auxiliary()
+      @auxiliary
+    end
+    # Get the mask of targets of meta search.
+    # The return value is the mask of targets of meta search.
+    def mask()
+      @mask
+    end
     #--------------------------------
     # private methods
     #--------------------------------
@@ -284,7 +357,10 @@ module EstraierPure
       @attrs = []
       @order = nil
       @max = -1
+      @skip = 0
       @options = 0
+      @auxiliary = 32
+      @mask = 0
     end
   end
   #----------------------------------------------------------------
@@ -300,8 +376,8 @@ module EstraierPure
     def uri()
       @uri
     end
-    # Get a list of attribute names.
-    # The return value is a list object of attribute names.
+    # Get an array of attribute names.
+    # The return value is an array object of attribute names.
     def attr_names()
       @attrs.keys.sort
     end
@@ -312,31 +388,31 @@ module EstraierPure
       @attrs[name]
     end
     # Get the snippet of a result document object.
-    # The return value is the snippet of the result document object.  There are tab separated
-    # values.  Each line is a string to be shown.  Though most lines have only one field, some
-    # lines have two fields.  If the second field exists, the first field is to be shown with
-    # highlighted, and the second field means its normalized form.
+    # The return value is a string of the snippet of the result document object.  There are tab
+    # separated values.  Each line is a string to be shown.  Though most lines have only one
+    # field, some lines have two fields.  If the second field exists, the first field is to be
+    # shown with highlighted, and the second field means its normalized form.
     def snippet()
       @snippet
     end
-    # Get the keyword vector.
-    # The return value is the keyword vector of the result document object.  There are tab
-    # separated values.  Keywords and their scores come alternately.
-    def vector()
-      @vector
+    # Get keywords.
+    # The return value is a string of serialized keywords of the result document object.  There
+    # are tab separated values.  Keywords and their scores come alternately.
+    def keywords()
+      @keywords
     end
     #--------------------------------
     # private methods
     #--------------------------------
     private
     # Create a result document object.
-    def initialize(uri, attrs, snippet, vector)
+    def initialize(uri, attrs, snippet, keywords)
       Utility::check_types({ uri=>String, attrs=>Hash,
-                             snippet=>String, vector=>String }) if $DEBUG
+                             snippet=>String, keywords=>String }) if $DEBUG
       @uri = uri
       @attrs = attrs
       @snippet = snippet
-      @vector = vector
+      @keywords = keywords
     end
   end
   #----------------------------------------------------------------
@@ -361,7 +437,7 @@ module EstraierPure
     end
     # Get the value of hint information.
     # `key' specifies the key of a hint.  "VERSION", "NODE", "HIT", "HINT#n", "DOCNUM", "WORDNUM",
-    # "TIME", "LINK#n", and "VIEW" are provided for keys.
+    # "TIME", "TIME#n", "LINK#n", and "VIEW" are provided for keys.
     # The return value is the hint or `nil' if the key does not exist.
     def hint(key)
       Utility::check_types({ key=>String }) if $DEBUG
@@ -426,13 +502,37 @@ module EstraierPure
     def status()
       @status
     end
+    # Synchronize updating contents of the database.
+    # The return value is true if success, else it is false.
+    def sync()
+      @status = -1
+      return false unless @url
+      turl = @url + "/sync"
+      reqheads = [ "Content-Type: application/x-www-form-urlencoded" ]
+      reqheads.push("Authorization: Basic " + Utility::base_encode(@auth)) if @auth
+      rv = Utility::shuttle_url(turl, @pxhost, @pxport, @timeout, reqheads, "", nil, nil)
+      @status = rv
+      rv == 200
+    end
+    # Optimize the database.
+    # The return value is true if success, else it is false.
+    def optimize()
+      @status = -1
+      return false unless @url
+      turl = @url + "/optimize"
+      reqheads = [ "Content-Type: application/x-www-form-urlencoded" ]
+      reqheads.push("Authorization: Basic " + Utility::base_encode(@auth)) if @auth
+      rv = Utility::shuttle_url(turl, @pxhost, @pxport, @timeout, reqheads, "", nil, nil)
+      @status = rv
+      rv == 200
+    end
     # Add a document.
     # `doc' specifies a document object.  The document object should have the URI attribute.
     # The return value is true if success, else it is false.
     def put_doc(doc)
       Utility::check_types({ doc=>Document }) if $DEBUG
       @status = -1
-      return false if !@url
+      return false unless @url
       turl = @url + "/put_doc"
       reqheads = [ "Content-Type: text/x-estraier-draft" ]
       reqheads.push("Authorization: Basic " + Utility::base_encode(@auth)) if @auth
@@ -447,7 +547,7 @@ module EstraierPure
     def out_doc(id)
       Utility::check_types({ id=>Integer }) if $DEBUG
       @status = -1
-      return false if !@url
+      return false unless @url
       turl = @url + "/out_doc"
       reqheads = [ "Content-Type: application/x-www-form-urlencoded" ]
       reqheads.push("Authorization: Basic " + Utility::base_encode(@auth)) if @auth
@@ -462,7 +562,7 @@ module EstraierPure
     def out_doc_by_uri(uri)
       Utility::check_types({ uri=>String }) if $DEBUG
       @status = -1
-      return false if !@url
+      return false unless @url
       turl = @url + "/out_doc"
       reqheads = [ "Content-Type: application/x-www-form-urlencoded" ]
       reqheads.push("Authorization: Basic " + Utility::base_encode(@auth)) if @auth
@@ -477,7 +577,7 @@ module EstraierPure
     def edit_doc(doc)
       Utility::check_types({ doc=>Document }) if $DEBUG
       @status = -1
-      return false if !@url
+      return false unless @url
       turl = @url + "/edit_doc"
       reqheads = [ "Content-Type: text/x-estraier-draft" ]
       reqheads.push("Authorization: Basic " + Utility::base_encode(@auth)) if @auth
@@ -492,7 +592,7 @@ module EstraierPure
     def get_doc(id)
       Utility::check_types({ id=>Integer }) if $DEBUG
       @status = -1
-      return nil if !@url
+      return nil unless @url
       turl = @url + "/get_doc"
       reqheads = [ "Content-Type: application/x-www-form-urlencoded" ]
       reqheads.push("Authorization: Basic " + Utility::base_encode(@auth)) if @auth
@@ -509,7 +609,7 @@ module EstraierPure
     def get_doc_by_uri(uri)
       Utility::check_types({ uri=>String }) if $DEBUG
       @status = -1
-      return nil if !@url
+      return nil unless @url
       turl = @url + "/get_doc"
       reqheads = [ "Content-Type: application/x-www-form-urlencoded" ]
       reqheads.push("Authorization: Basic " + Utility::base_encode(@auth)) if @auth
@@ -527,7 +627,7 @@ module EstraierPure
     def get_doc_attr(id, name)
       Utility::check_types({ id=>Integer, name=>String }) if $DEBUG
       @status = -1
-      return nil if !@url
+      return nil unless @url
       turl = @url + "/get_doc_attr"
       reqheads = [ "Content-Type: application/x-www-form-urlencoded" ]
       reqheads.push("Authorization: Basic " + Utility::base_encode(@auth)) if @auth
@@ -545,7 +645,7 @@ module EstraierPure
     def get_doc_attr_by_uri(uri, name)
       Utility::check_types({ uri=>String, name=>String }) if $DEBUG
       @status = -1
-      return nil if !@url
+      return nil unless @url
       turl = @url + "/get_doc_attr"
       reqheads = [ "Content-Type: application/x-www-form-urlencoded" ]
       reqheads.push("Authorization: Basic " + Utility::base_encode(@auth)) if @auth
@@ -563,7 +663,7 @@ module EstraierPure
     def etch_doc(id)
       Utility::check_types({ id=>Integer }) if $DEBUG
       @status = -1
-      return nil if !@url
+      return nil unless @url
       turl = @url + "/etch_doc"
       reqheads = [ "Content-Type: application/x-www-form-urlencoded" ]
       reqheads.push("Authorization: Basic " + Utility::base_encode(@auth)) if @auth
@@ -573,7 +673,7 @@ module EstraierPure
       @status = rv
       return nil if rv != 200
       kwords = {}
-      lines = resbody.string.split(/\n/)
+      lines = resbody.string.split(/\n/, -1)
       for i in 0...lines.length
         pair = lines[i].split(/\t/)
         next if pair.length < 2
@@ -588,17 +688,17 @@ module EstraierPure
     def etch_doc_by_uri(uri)
       Utility::check_types({ uri=>String }) if $DEBUG
       @status = -1
-      return nil if !@url
+      return nil unless @url
       turl = @url + "/etch_doc"
       reqheads = [ "Content-Type: application/x-www-form-urlencoded" ]
       reqheads.push("Authorization: Basic " + Utility::base_encode(@auth)) if @auth
-      reqbody = "uri=" + URI::encode(uri);
+      reqbody = "uri=" + URI::encode(uri)
       resbody = StringIO::new
       rv = Utility::shuttle_url(turl, @pxhost, @pxport, @timeout, reqheads, reqbody, nil, resbody)
       @status = rv
       return nil if rv != 200
       kwords = {}
-      lines = resbody.string.split(/\n/)
+      lines = resbody.string.split(/\n/, -1)
       for i in 0...lines.length
         pair = lines[i].split(/\t/)
         next if pair.length < 2
@@ -612,7 +712,7 @@ module EstraierPure
     def uri_to_id(uri)
       Utility::check_types({ uri=>String }) if $DEBUG
       @status = -1
-      return -1 if !@url
+      return -1 unless @url
       turl = @url + "/uri_to_id"
       reqheads = [ "Content-Type: application/x-www-form-urlencoded" ]
       reqheads.push("Authorization: Basic " + Utility::base_encode(@auth)) if @auth
@@ -653,14 +753,48 @@ module EstraierPure
       set_info if @size < 0.0
       @size
     end
-    # Search documents corresponding a condition.
+    # Get the usage ratio of the cache.
+    # The return value is the usage ratio of the cache.  On error, -1.0 is returned.
+    def cache_usage()
+      @status = -1
+      return -1.0 unless @url
+      turl = @url + "/cacheusage"
+      reqheads = []
+      reqheads.push("Authorization: Basic " + Utility::base_encode(@auth)) if @auth
+      resbody = StringIO::new
+      rv = Utility::shuttle_url(turl, @pxhost, @pxport, @timeout, reqheads, nil, nil, resbody)
+      @status = rv
+      return -1.0 if rv != 200
+      return resbody.string.strip.to_f
+    end
+    # Get an array of names of administrators.
+    # The return value is an array object of names of administrators.  On error, `nil' is
+    # returned.
+    def admins()
+      set_info unless @admins
+      @admins
+    end
+    # Get an array of names of users.
+    # The return value is an array object of names of users.  On error, `nil' is returned.
+    def users()
+      set_info unless @users
+      @users
+    end
+    # Get an array of expressions of links.
+    # The return value is an array object of expressions of links.  Each element is a TSV string
+    # and has three fields of the URL, the label, and the score.  On error, `nil' is returned.
+    def links()
+      set_info unless @links
+      @links
+    end
+    # Search for documents corresponding a condition.
     # `cond' specifies a condition object.
     # `depth' specifies the depth of meta search.
     # The return value is a node result object.  On error, `nil' is returned.
     def search(cond, depth)
       Utility::check_types({ cond=>Condition, depth=>Integer }) if $DEBUG
       @status = -1
-      return nil if !@url
+      return nil unless @url
       turl = @url + "/search"
       reqheads = [ "Content-Type: application/x-www-form-urlencoded" ]
       reqheads.push("Authorization: Basic " + Utility::base_encode(@auth)) if @auth
@@ -669,7 +803,7 @@ module EstraierPure
       rv = Utility::shuttle_url(turl, @pxhost, @pxport, @timeout, reqheads, reqbody, nil, resbody)
       @status = rv
       return nil if rv != 200
-      lines = resbody.string.split(/\n/)
+      lines = resbody.string.split(/\n/, -1)
       return nil if lines.length < 1
       docs = []
       hints = {}
@@ -702,12 +836,12 @@ module EstraierPure
             rdvector = ""
             rlnum = snum
             while rlnum < lnum - 1
-              rdline = lines[rlnum].lstrip.rstrip
+              rdline = lines[rlnum].strip
               rlnum += 1
               break if rdline.length < 1
               if rdline =~ /^%/
                 lidx = rdline.index("\t")
-                rdvector = rdline[(lidx+1)...rdline.length] if rdline =~ /%VECTOR/
+                rdvector = rdline[(lidx+1)...rdline.length] if rdline =~ /%VECTOR/ && lidx
               else
                 lidx = rdline.index("=")
                 if lidx
@@ -744,9 +878,9 @@ module EstraierPure
     # `awidth' specifies width of strings picked up around each highlighted word. By default,
     # it is 96.  If it is negative, the current setting is not changed.
     def set_snippet_width(wwidth, hwidth, awidth)
-      @wwidth = wwidth;
-      @hwidth = hwidth if(hwidth >= 0)
-      @awidth = awidth if(awidth >= 0)
+      @wwidth = wwidth
+      @hwidth = hwidth if hwidth >= 0
+      @awidth = awidth if awidth >= 0
     end
     # Manage a user account of a node.
     # `name' specifies the name of a user.
@@ -756,7 +890,7 @@ module EstraierPure
     def set_user(name, mode)
       Utility::check_types({ name=>String, mode=>Integer }) if $DEBUG
       @status = -1
-      return false if !@url
+      return false unless @url
       turl = @url + "/_set_user"
       reqheads = [ "Content-Type: application/x-www-form-urlencoded" ]
       reqheads.push("Authorization: Basic " + Utility::base_encode(@auth)) if @auth
@@ -773,7 +907,7 @@ module EstraierPure
     def set_link(url, label, credit)
       Utility::check_types({ url=>String, label=>String, credit=>Integer }) if $DEBUG
       @status = -1
-      return false if !@url
+      return false unless @url
       turl = @url + "/_set_link"
       reqheads = [ "Content-Type: application/x-www-form-urlencoded" ]
       reqheads.push("Authorization: Basic " + Utility::base_encode(@auth)) if @auth
@@ -799,15 +933,18 @@ module EstraierPure
       @dnum = -1
       @wnum = -1
       @size = -1.0
-      @wwidth = 480;
-      @hwidth = 96;
-      @awidth = 96;
+      @admins = nil
+      @users = nil
+      @links = nil
+      @wwidth = 480
+      @hwidth = 96
+      @awidth = 96
       @status = -1
     end
     # Set information of the node.
     def set_info()
       @status = -1
-      return if !@url
+      return unless @url
       turl = @url + "/inform"
       reqheads = []
       reqheads.push("Authorization: Basic " + Utility::base_encode(@auth)) if @auth
@@ -815,7 +952,7 @@ module EstraierPure
       rv = Utility::shuttle_url(turl, @pxhost, @pxport, @timeout, reqheads, nil, nil, resbody)
       @status = rv
       return if rv != 200
-      lines = resbody.string.chomp.split(/\n/)
+      lines = resbody.string.split(/\n/, -1)
       return if lines.length < 1
       elems = lines[0].chomp.split(/\t/)
       return if elems.length != 5
@@ -824,6 +961,32 @@ module EstraierPure
       @dnum = elems[2].to_i
       @wnum = elems[3].to_i
       @size = elems[4].to_f
+      return if lines.length < 2
+      lnum = 1
+      lnum += 1 if(lnum < lines.length && lines[lnum].length < 1)
+      @admins = []
+      while(lnum < lines.length)
+        line = lines[lnum]
+        break if line.length < 1
+        @admins.push(line)
+        lnum += 1
+      end
+      lnum += 1 if(lnum < lines.length && lines[lnum].length < 1)
+      @users = []
+      while(lnum < lines.length)
+        line = lines[lnum]
+        break if line.length < 1
+        @users.push(line)
+        lnum += 1
+      end
+      lnum += 1 if(lines[lnum].length < 1)
+      @links = []
+      while(lnum < lines.length)
+        line = lines[lnum]
+        break if line.length < 1
+        @links.push(line) if line.split(/\t/).length == 3
+        lnum += 1
+      end
     end
   end
   #:stopdoc:
@@ -851,10 +1014,10 @@ module EstraierPure
     # `pxhost' specifies the host name of a proxy.  If it is `nil', it is not used.
     # `pxport' specifies the port number of the proxy.
     # `outsec' specifies timeout in seconds.  If it is negative, it is not used.
-    # `reqheads' specifies a list object of extension headers.  If it is `nil', it is not used.
+    # `reqheads' specifies an array object of extension headers.  If it is `nil', it is not used.
     # `reqbody' specifies the pointer of the entitiy body of request.  If it is `nil', "GET"
     # method is used.
-    # `resheads' specifies a list object into which headers of response is stored.  If it is
+    # `resheads' specifies an array object into which headers of response is stored.  If it is
     # `nil' it is not used.
     # `resbody' specifies stream object into which the entity body of response is stored.  If it
     # is `nil', it is not used.
@@ -949,7 +1112,7 @@ module EstraierPure
         buf.write("order=")
         buf.write(URI::encode(cond.order))
       end
-      if cond.max > 0
+      if cond.max >= 0
         buf.write("&") if buf.length > 0
         buf.write("max=" + cond.max.to_s)
       else
@@ -957,10 +1120,13 @@ module EstraierPure
         buf.write("max=" + (1 << 30).to_s)
       end
       buf.write("&options=" + cond.options.to_s) if cond.options > 0
+      buf.write("&auxiliary=" + cond.auxiliary.to_s)
       buf.write("&depth=" + depth.to_s) if depth > 0
       buf.write("&wwidth=" + wwidth.to_s)
       buf.write("&hwidth=" + hwidth.to_s)
       buf.write("&awidth=" + awidth.to_s)
+      buf.write("&skip=" + cond.skip.to_s)
+      buf.write("&mask=" + cond.mask.to_s)
       buf.string
     end
     module_function :cond_to_query
