@@ -820,7 +820,7 @@ int fetch_document(const char *url, const char *pxhost, int pxport, int outsec,
   if(raw){
     rbuf = NULL;
   } else {
-    rbuf = cbdatumopen("", 0);
+    rbuf = cbdatumopen(NULL, -1);
     raw = rbuf;
   }
   if(heads){
@@ -1037,7 +1037,7 @@ static void make_doc_from_draft(const char *buf, int size, ESTDOC *doc, CBLIST *
   assert(buf && size >= 0 && doc);
   lines = cbsplit(buf, -1, "\n");
   for(i = 0; i < CB_LISTNUM(lines); i++){
-    line = CB_LISTVAL(lines, i, NULL);
+    line = CB_LISTVAL(lines, i);
     while(*line > '\0' && *line <= ' '){
       line++;
     }
@@ -1051,7 +1051,7 @@ static void make_doc_from_draft(const char *buf, int size, ESTDOC *doc, CBLIST *
     }
   }
   for(; i < CB_LISTNUM(lines); i++){
-    line = CB_LISTVAL(lines, i, NULL);
+    line = CB_LISTVAL(lines, i);
     if(*line == '\t'){
       est_doc_add_hidden_text(doc, line + 1);
     } else {
@@ -1088,23 +1088,23 @@ static void make_doc_from_text(const char *buf, int size, const char *penc, int
     if(nbuf) text = nbuf;
   }
   lines = cbsplit(text, -1, "\n");
-  datum = cbdatumopen("", 0);
+  CB_DATUMOPEN(datum);
   for(i = 0; i < CB_LISTNUM(lines); i++){
-    line = CB_LISTVAL(lines, i, NULL);
+    line = CB_LISTVAL(lines, i);
     while(*line == ' ' || *line == '\t' || *line == '\r'){
       line++;
     }
     if(line[0] == '\0'){
       est_doc_add_text(doc, CB_DATUMPTR(datum));
-      cbdatumsetsize(datum, 0);
+      CB_DATUMSETSIZE(datum, 0);
     } else {
-      cbdatumcat(datum, " ", 1);
-      cbdatumcat(datum, line, -1);
+      CB_DATUMCAT(datum, " ", 1);
+      CB_DATUMCAT(datum, line, strlen(line));
     }
   }
   est_doc_add_text(doc, CB_DATUMPTR(datum));
-  cbdatumclose(datum);
-  cblistclose(lines);
+  CB_DATUMCLOSE(datum);
+  CB_LISTCLOSE(lines);
   est_doc_add_attr(doc, ESTDATTRTYPE, "text/plain");
   sprintf(numbuf, "%d", size);
   est_doc_add_attr(doc, ESTDATTRSIZE, numbuf);
@@ -1212,10 +1212,10 @@ static void make_doc_from_html(const char *buf, int size, const char *penc, int
   }
   if(nbuf) html = nbuf;
   if(!html) html = buf;
-  datum = cbdatumopen("", 0);
+  CB_DATUMOPEN(datum);
   elems = cbxmlbreak(html, TRUE);
   for(i = 0; i < CB_LISTNUM(elems); i++){
-    elem = CB_LISTVAL2(elems, i, &esiz);
+    elem = CB_LISTVAL2(elems, i, esiz);
     if(!(next = cblistval(elems, i + 1, NULL))) next = "";
     if(elem[0] == '<'){
       if(cbstrfwimatch(elem, "<html")){
@@ -1314,7 +1314,7 @@ static void make_doc_from_html(const char *buf, int size, const char *penc, int
         } else {
           est_doc_add_text(doc, CB_DATUMPTR(datum));
         }
-        cbdatumsetsize(datum, 0);
+        CB_DATUMSETSIZE(datum, 0);
       } else if(links && (cbstrfwimatch(elem, "<a") || cbstrfwimatch(elem, "<link"))){
         attrs = cbxmlattrs(elem);
         value = cbmapget(attrs, "href", -1, NULL);
@@ -1333,11 +1333,11 @@ static void make_doc_from_html(const char *buf, int size, const char *penc, int
         cbmapclose(attrs);
       }
     } else {
-      cbdatumcat(datum, " ", -1);
-      cbdatumcat(datum, elem, esiz);
+      CB_DATUMCAT(datum, " ", 1);
+      CB_DATUMCAT(datum, elem, esiz);
     }
   }
-  cblistclose(elems);
+  CB_LISTCLOSE(elems);
   if(strchr(CB_DATUMPTR(datum), '&')){
     rbuf = html_raw_text(CB_DATUMPTR(datum));
     est_doc_add_text(doc, rbuf);
@@ -1345,7 +1345,7 @@ static void make_doc_from_html(const char *buf, int size, const char *penc, int
   } else {
     est_doc_add_text(doc, CB_DATUMPTR(datum));
   }
-  cbdatumclose(datum);
+  CB_DATUMCLOSE(datum);
   if(nbuf) free(nbuf);
   est_doc_add_attr(doc, ESTDATTRTYPE, "text/html");
   sprintf(numbuf, "%d", size);
@@ -1367,7 +1367,7 @@ static char *html_enc(const char *str){
   assert(str);
   elems = cbxmlbreak(str, TRUE);
   for(i = 0; i < CB_LISTNUM(elems); i++){
-    elem = CB_LISTVAL(elems, i, NULL);
+    elem = CB_LISTVAL(elems, i);
     if(elem[0] != '<' || !cbstrfwimatch(elem, "<meta")) continue;
     enc = NULL;
     attrs = cbxmlattrs(elem);
@@ -1389,11 +1389,11 @@ static char *html_enc(const char *str){
     }
     cbmapclose(attrs);
     if(enc){
-      cblistclose(elems);
+      CB_LISTCLOSE(elems);
       return enc;
     }
   }
-  cblistclose(elems);
+  CB_LISTCLOSE(elems);
   return NULL;
 }
 
@@ -1591,7 +1591,7 @@ static void make_doc_from_mime(const char *buf, int size, const char *penc, int
     if((bound = cbmapget(attrs, "BOUNDARY", -1, NULL)) != NULL){
       parts = cbmimeparts(body, bsiz, bound);
       for(i = 0; i < CB_LISTNUM(parts) && i < 8; i++){
-        part = CB_LISTVAL2(parts, i, &psiz);
+        part = CB_LISTVAL2(parts, i, psiz);
         tdoc = est_doc_new();
         make_doc_from_mime(part, psiz, penc, plang, tdoc, links);
         if(mht){
@@ -1602,12 +1602,12 @@ static void make_doc_from_mime(const char *buf, int size, const char *penc, int
         }
         texts = est_doc_texts(tdoc);
         for(j = 0; j < CB_LISTNUM(texts); j++){
-          text = CB_LISTVAL(texts, j, NULL);
+          text = CB_LISTVAL(texts, j);
           est_doc_add_text(doc, text);
         }
         est_doc_delete(tdoc);
       }
-      cblistclose(parts);
+      CB_LISTCLOSE(parts);
     }
   } else {
     key = cbmapget(attrs, "content-transfer-encoding", -1, NULL);
@@ -1647,23 +1647,23 @@ static void make_doc_from_mime(const char *buf, int size, const char *penc, int
           bsiz = ssiz;
         }
         lines = cbsplit(body, bsiz, "\n");
-        datum = cbdatumopen("", 0);
+        CB_DATUMOPEN(datum);
         for(i = 0; i < CB_LISTNUM(lines); i++){
-          line = CB_LISTVAL(lines, i, NULL);
+          line = CB_LISTVAL(lines, i);
           while(*line == ' ' || *line == '>' || *line == '|' || *line == '\t' || *line == '\r'){
             line++;
           }
           if(line[0] == '\0'){
             est_doc_add_text(doc, CB_DATUMPTR(datum));
-            cbdatumsetsize(datum, 0);
+            CB_DATUMSETSIZE(datum, 0);
           } else {
-            cbdatumcat(datum, " ", 1);
-            cbdatumcat(datum, line, -1);
+            CB_DATUMCAT(datum, " ", 1);
+            CB_DATUMCAT(datum, line, strlen(line));
           }
         }
         est_doc_add_text(doc, CB_DATUMPTR(datum));
-        cbdatumclose(datum);
-        cblistclose(lines);
+        CB_DATUMCLOSE(datum);
+        CB_LISTCLOSE(lines);
       }
     } else if(cbstrfwimatch(key, "text/html")){
       tdoc = est_doc_new();
@@ -1678,7 +1678,7 @@ static void make_doc_from_mime(const char *buf, int size, const char *penc, int
       }
       texts = est_doc_texts(tdoc);
       for(i = 0; i < CB_LISTNUM(texts); i++){
-        text = CB_LISTVAL(texts, i, NULL);
+        text = CB_LISTVAL(texts, i);
         est_doc_add_text(doc, text);
       }
       est_doc_delete(tdoc);
@@ -1695,7 +1695,7 @@ static void make_doc_from_mime(const char *buf, int size, const char *penc, int
       }
       texts = est_doc_texts(tdoc);
       for(i = 0; i < CB_LISTNUM(texts); i++){
-        text = CB_LISTVAL(texts, i, NULL);
+        text = CB_LISTVAL(texts, i);
         est_doc_add_text(doc, text);
       }
       est_doc_delete(tdoc);
@@ -1704,7 +1704,7 @@ static void make_doc_from_mime(const char *buf, int size, const char *penc, int
       make_doc_from_text(body, bsiz, penc, plang, tdoc, links);
       texts = est_doc_texts(tdoc);
       for(i = 0; i < CB_LISTNUM(texts); i++){
-        text = CB_LISTVAL(texts, i, NULL);
+        text = CB_LISTVAL(texts, i);
         est_doc_add_text(doc, text);
       }
       est_doc_delete(tdoc);
